package cn.iocoder.yudao.module.product.dal.dataobject.coupon;

import io.swagger.annotations.ApiModelProperty;
import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import com.baomidou.mybatisplus.annotation.*;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;

/**
 * 优惠券信息 DO
 *
 * @author 系统管理员
 */
@TableName("ecw_coupon")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class CouponDO extends BaseDO {

    /**
     * 优惠券ID
     */
    @TableId
    private Long couponId;
    /**
     * 标题-中文
     */
    private String titleZh;
    /**
     * 标题-英文
     */
    private String titleEn;
    /**
     * 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5优惠 6 特价 7区间价
     */
    private Integer type;
    /**
     * 使用条件(优惠卷专用)
     */
    private BigDecimal cashCondition;
    /**
     * 减免金额(优惠卷专用)
     */
    private BigDecimal reduceAmount;
    /**
     * 条件货币ID
     */
    private Long conditionCurrencyId;
    /**
     * 开始时间
     */
    private Date startTime;
    /**
     * 减免货币ID
     */
    private Long reduceCurrencyId;
    /**
     * 结束时间
     */
    private Date endTime;
    /**
     * 领券后X天起生效
     */
    private Integer afterReceiveDays;
    /**
     * 有效天数
     */
    private Integer validDays;
    /**
     * 费用类型：1 运输费 2 清关费 3 总费用(优惠卷专用)
     */
    private Integer costType;
    /**
     * 优惠清单
     */
    private String discountDetailed;
    /**
     * 同类优惠劵是否可以叠加 0否  1是
     */
    private Boolean isSimilarSuperposition;
    /**
     * 可叠加优惠劵列表
     */
    private String couponIds;
    /**
     * 适用用户类型 0全部用户 1部分用户 2指定用户不参与 3 注册即送
     */
    private Integer suitableUserType;
    /**
     * 适用线路类型 0全部线路参与 1指定线路参与 2指定线路不参与
     */
    private Integer suitableLineType;
    /**
     * 适用产品类型 0全部产品参与 1指定产品参与 2指定产品不参与
     */
    private Integer suitableProdType;
    /**
     * 每个用户领券上限，如不填则默认为1
     */
    private Integer limitNum;
    /**
     * 优惠券过期状态 0:过期 1:未过期
     */
    private Boolean overdueStatus;
    /**
     * 优惠券投放状态(-1:取消投放 0:等待投放 1:投放 2:违规下架 3:等待审核)
     */
    private Integer putonStatus;
    /**
     * 获取方式 0=客户领取 1=平台发放
     */
    private Integer getWay = 1;
    /**
     * 订单属性(可多选) 1 普货 2重货 3泡货
     */
    private String orderAttr;
    /**
     * 是否发送站内信 0否  1是
     */
    private Boolean isSiteContent;
    /**
     * 品牌(可多选): 0 无牌价 1 有牌价 2 中性价
     */
    private String brand;
    /**
     * 是否控货(可多选) 0否 1是
     */
    private String isCargoControl;
    /**
     * 单证报关(可多选) 1 我司全代 2自单代报 3混合报关
     */
    private String documentDeclaration;
    /**
     * 优惠券状态(0:草稿 1:发布 2:过期)
     */
    private Integer status;


    @ApiModelProperty(value = "创建者名称")
    @TableField(exist = false)
    private String creatorName;

    @ApiModelProperty(value = "更新者名称")
    @TableField(exist = false)
    private String updaterName;
}
