package cn.iocoder.yudao.module.product.dal.dataobject.product;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.util.List;

@Data
public class BasePrice extends BaseDO {

    /**
     * 运费
     */
    private BigDecimal transportPrice;
    /**
     * 运费阶梯价格主键id
     */
    @TableField(exist = false)
    @JsonIgnore
    private Long transportStepPriceId;
    /**
     * 运费金额单位
     */
    private Long transportPriceUnit;
    /**
     * 运费体积单位
     */
    private Long transportVolumeUnit;
    /**
     * 清关费用
     */
    private BigDecimal clearancePrice;
    /**
     * 清关费阶梯价格汇率换算为美元后的额度
     */
    @TableField(exist = false)
    @JsonIgnore
    private BigDecimal productClearanceAsExchangePrice;
    /**
     * 清关费阶梯价格主键id
     */
    @TableField(exist = false)
    @JsonIgnore
    private Long clearanceStepPriceId;
    /**
     * 清关金额单位
     */
    private Long clearancePriceUnit;
    /**
     * 清关体积单位
     */
    private Long clearanceVolumeUnit;
    /**
     * 全包价金额
     */
    private BigDecimal allPrice;
    /**
     * 全包阶梯价格主键id
     */
    @TableField(exist = false)
    @JsonIgnore
    private Long allStepPriceId;
    /**
     * 全包价金额单位
     */
    private Long allPriceUnit;

    /**
     * 全包价体积单位
     */
    private Long allVolumeUnit;


    @ApiModelProperty(value = "空运清关费来源：1 商品 2 渠道")
    @TableField(exist = false)
    @JsonIgnore
    private Integer airClearanceSource;

    @ApiModelProperty(value = "空运清关费来源数据详情-->数据详情json字符串")
    @TableField(exist = false)
    @JsonIgnore
    private String airClearanceInfo;

    //1-未设置商品清关费, 2-区间未匹配上
    /**
     * 1-未设置商品清关费, 2-区间未匹配上
     */
    @TableField(exist = false)
    @JsonIgnore
    private int productPriceEmptyType = 0;


    @ApiModelProperty(value = "特殊需求列表")
    @TableField(exist = false)
    private List<ProductPriceSpecialDO> specialDOList;

    @ApiModelProperty(value = "运费包装价格列表")
    @TableField(exist = false)
    private List<ProductPricePackagingDO> freightPricePackagingList;

    @ApiModelProperty(value = "清关费包装价格列表")
    @TableField(exist = false)
    private List<ProductPricePackagingDO> clearancePricePackagingList;

    @ApiModelProperty(value = "全包价包装价格列表")
    @TableField(exist = false)
    private List<ProductPricePackagingDO> fullPricePackagingList;

}
