package cn.iocoder.yudao.module.product.dal.mysql.coupon;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponUseRecordDO;
import cn.iocoder.yudao.module.product.vo.coupon.customer.CouponCustomerExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.customer.CouponCustomerInfoVO;
import cn.iocoder.yudao.module.product.vo.coupon.customer.CouponCustomerPageReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponCustomerDO;
import cn.iocoder.yudao.module.product.vo.coupon.customer.CouponCustomerRespVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordRespVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 优惠券客户关联信息 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface CouponCustomerMapper extends BaseMapperX<CouponCustomerDO> {

    default PageResult<CouponCustomerDO> selectPage(CouponCustomerPageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<CouponCustomerDO>()
                .eqIfPresent(CouponCustomerDO::getCouponId, reqVO.getCouponId())
                .eqIfPresent(CouponCustomerDO::getUserId, reqVO.getUserId())
                .eqIfPresent(CouponCustomerDO::getCustomerId, reqVO.getCustomerId())
//                .betweenIfPresent(CouponCustomerDO::getReceiveTime, reqVO.getBeginReceiveTime(), reqVO.getEndReceiveTime())
                .eqIfPresent(CouponCustomerDO::getStatus, reqVO.getStatus())
//                .betweenIfPresent(CouponCustomerDO::getUserStartTime, reqVO.getBeginUserStartTime(), reqVO.getEndUserStartTime())
//                .betweenIfPresent(CouponCustomerDO::getUserEndTime, reqVO.getBeginUserEndTime(), reqVO.getEndUserEndTime())
                .eqIfPresent(CouponCustomerDO::getIsDelete, reqVO.getIsDelete())
                .orderByDesc(CouponCustomerDO::getCouponCustomerId));
    }

    default List<CouponCustomerDO> selectList(CouponCustomerExportReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<CouponCustomerDO>()
                .eqIfPresent(CouponCustomerDO::getCouponId, reqVO.getCouponId())
                .eqIfPresent(CouponCustomerDO::getUserId, reqVO.getUserId())
                .eqIfPresent(CouponCustomerDO::getCustomerId, reqVO.getCustomerId())
                .betweenIfPresent(CouponCustomerDO::getReceiveTime, reqVO.getBeginReceiveTime(), reqVO.getEndReceiveTime())
                .eqIfPresent(CouponCustomerDO::getStatus, reqVO.getStatus())
                .betweenIfPresent(CouponCustomerDO::getUserStartTime, reqVO.getBeginUserStartTime(), reqVO.getEndUserStartTime())
                .betweenIfPresent(CouponCustomerDO::getUserEndTime, reqVO.getBeginUserEndTime(), reqVO.getEndUserEndTime())
                .eqIfPresent(CouponCustomerDO::getIsDelete, reqVO.getIsDelete())
                .orderByDesc(CouponCustomerDO::getCouponCustomerId));
    }

    @ResultType(Long.class)
    @Select({"SELECT customer_id FROM ecw_coupon_customer WHERE coupon_id = #{couponId}"})
    List<Long> getCustomerIdsByCouponId(@Param("couponId") Long couponId);

    @ResultType(CouponCustomerInfoVO.class)
    @Select({
            "<script>",
            "SELECT c.*, ",
            "cc.* ",
            "FROM ecw_coupon_customer cc ",
            "LEFT JOIN ecw_coupon c ON cc.coupon_id = c.coupon_id ",
            " WHERE ",
            " cc.is_delete = 0",
            "<when test = 'customerId != null'> ",
            "AND cc.customer_id = #{customerId} ",
            "</when> ",
            "<when test = 'couponCustomerIds != null'> ",
            "cc.coupon_customer_id in ",
            "<foreach item='couponCustomerId' index='index' collection='couponCustomerIds' open='(' separator=',' close=')'>#{couponCustomerId}</foreach> ",
            "</when> ",
            "</script>"
    })
    List<CouponCustomerInfoVO> listInfo(@Param("couponCustomerIds") List<String> couponCustomerIds, @Param("customerId") Long customerId);

    @ResultType(CouponCustomerRespVO.class)
    @Select({
            "<script>",
            "select ",
            "cc.*,",
            "c.* ",
            "from ecw_coupon_customer cc ",
            "left join ecw_coupon c ",
            "on cc.coupon_id = c.coupon_id ",
            "where c.type = 1 ",
            "and cc.user_id = #{reqVo.userId}",
            "<when test = 'reqVo.status != null'> ",
            "AND cc.status = #{reqVo.status} ",
            "</when> ",
            "</script>"
    })
    IPage<CouponCustomerRespVO> couponUseRecordPage(@Param("mpPage") IPage<CouponCustomerRespVO> mpPage,
                                                    @Param("reqVo") CouponCustomerPageReqVO reqVO);
}
