package cn.iocoder.yudao.module.product.dal.mysql.product;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrExportReqVO;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrPageReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 产品属性 Mapper
 *
 * @author wlh
 */
@Mapper
public interface ProductAttrMapper extends BaseMapperX<ProductAttrDO> {

    default PageResult<ProductAttrDO> selectPage(ProductAttrPageReqVO reqVO) {
        String keywords = reqVO.getAttrName();
        boolean hasKeywords = StringUtils.isNotEmpty(keywords);
        return selectPage(reqVO, new LambdaQueryWrapperX<ProductAttrDO>()
                .like(hasKeywords, ProductAttrDO::getAttrName, keywords)
                .or()
                .like(hasKeywords, ProductAttrDO::getAttrCode, keywords)
                .orderByAsc(ProductAttrDO::getSort));
    }

    default List<ProductAttrDO> selectList(ProductAttrExportReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<ProductAttrDO>()
                .likeIfPresent(ProductAttrDO::getAttrName, reqVO.getAttrName())
                .eqIfPresent(ProductAttrDO::getAttrCode, reqVO.getAttrCode())
                .eqIfPresent(ProductAttrDO::getRemark, reqVO.getRemark())
                .eqIfPresent(ProductAttrDO::getStatus, reqVO.getStatus())
                .betweenIfPresent(ProductAttrDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByAsc(ProductAttrDO::getSort));
    }

    @ResultType(Long.class)
    @Select({
            "select `id` from ecw_product_attr where `status` = 0 and deleted = 0"
    })
    List<Long> getAllAttrId();

}
