package cn.iocoder.yudao.module.product.dto;

import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.param.ProdConditionParam;
import cn.iocoder.yudao.module.product.vo.coupon.CouponInfoVO;
import cn.iocoder.yudao.module.product.vo.coupon.DiscountDetailedVO;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Data
@ApiModel(value = "可用优惠劵")
public class CouponAvailableDto {

    @ApiModelProperty(value = "优惠券ID", required = true)
    private Long couponId;

    @ApiModelProperty(value = "标题-中文")
    private String titleZh;

    @ApiModelProperty(value = "标题-英文")
    private String titleEn;

    @ApiModelProperty(value = "类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5优惠 6 特价 7区间价")
    private Integer type;

    @ApiModelProperty(value = "使用条件(优惠卷专用)")
    private BigDecimal cashCondition = BigDecimal.ZERO;

    @ApiModelProperty(value = "减免金额")
    private BigDecimal reduceAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "减免总金额")
    private BigDecimal reduceTotalAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "条件货币ID")
    private Integer conditionCurrencyId;

    @ApiModelProperty(value = "减免货币ID")
    private Integer reduceCurrencyId;

    @ApiModelProperty(value = "清关费减免金额")
    private BigDecimal clearanceReduceAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "清关费减免总金额")
    private BigDecimal clearanceReduceTotalAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "清关费条件货币ID")
    private Integer clearanceConditionCurrencyId;

    @ApiModelProperty(value = "清关费减免货币ID")
    private Integer clearanceReduceCurrencyId;

    @ApiModelProperty(value = "总计人民币优惠金额")
    private BigDecimal totalRMBReduceAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "开始时间")
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date startTime;

    @ApiModelProperty(value = "结束时间")
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date endTime;

    @ApiModelProperty(value = "领券后X天起生效")
    private Integer afterReceiveDays;

    @ApiModelProperty(value = "有效天数")
    private Integer validDays;

    @ApiModelProperty(value = "费用类型：1 运输费 2 清关费 3 总费用(优惠卷专用)")
    private Integer costType;

    @ApiModelProperty(value = "优惠清单")
    private String discountDetailed;

    @ApiModelProperty(value = "优惠清单")
    private List<DiscountDetailedVO> discountDetailedVOList;

    @ApiModelProperty(value = "同类优惠劵是否不能叠加 0否  1是")
    private Boolean isSimilarSuperposition;

    @ApiModelProperty(value = "可叠加优惠劵列表")
    private String couponIds;

    @ApiModelProperty(value = "适用用户类型 0全部用户 1部分用户 2指定用户不参与 3 注册即送")
    private Integer suitableUserType;

    @ApiModelProperty(value = "适用线路类型 0全部线路参与 1指定线路参与 2指定线路不参与")
    private Integer suitableLineType;

    @ApiModelProperty(value = "适用产品类型 0全部产品参与 1指定产品参与 2指定产品不参与")
    private Integer suitableProdType;

    @ApiModelProperty(value = "每个用户领券上限，如不填则默认为1")
    private Integer limitNum;

    @ApiModelProperty(value = "优惠券过期状态 0:过期 1:未过期")
    private Boolean overdueStatus;

    @ApiModelProperty(value = "优惠券投放状态(-1:取消投放 0:等待投放 1:投放 2:违规下架 3:等待审核)")
    private Integer putonStatus;

    @ApiModelProperty(value = "获取方式 0=客户领取 1=平台发放")
    private Integer getWay;

    @ApiModelProperty(value = "订单属性(可多选) 1 普货 2重货 3泡货")
    private String orderAttr;

    @ApiModelProperty(value = "是否发送站内信 0否  1是")
    private Boolean isSiteContent;

    @ApiModelProperty(value = "品牌(可多选): 0 无牌价 1 有牌价 2 中性价")
    private String brand;

    @ApiModelProperty(value = "是否控货(可多选) 0否 1是")
    private String isCargoControl;

    @ApiModelProperty(value = "单证报关(可多选) 1 我司全代 2自单代报 3混合报关")
    private String documentDeclaration;

    @ApiModelProperty(value = "优惠券状态(0:草稿 1:发布)")
    private Integer status;

    @ApiModelProperty(value = "用户优惠卷状态")
    private Integer useStatus;

    @ApiModelProperty(value = "最大优惠(除优惠卷之外的优惠活动的最大优惠)")
    private DiscountDetailedVO discountDetailedVO;

    @ApiModelProperty(value = "包含的商品列表")
    private List<ProductDO> prodList;

    @ApiModelProperty(value = "是否默认")
    private Boolean isDefault = Boolean.FALSE;


    @ApiModelProperty(value = "优惠券ID", required = true)
    private Long prodId;

    @ApiModelProperty(value = "商品名称-中文")
    private String prodTitleZh;

    @ApiModelProperty(value = "商品名称-英文")
    private String prodTitleEn;

    @ApiModelProperty(value = "运费优惠详情")
    private DiscountDetailedVO freightDetailed;

    @ApiModelProperty(value = "清关费优惠详情")
    private DiscountDetailedVO clearanceDetailed;

    @ApiModelProperty(value = "使用的优惠详情列表")
    private List<CouponInfoVO> couponInfoVOList;


    private BigDecimal couponTotalChargeVolume = BigDecimal.ZERO;

    private BigDecimal couponTotalChargeWeight = BigDecimal.ZERO;

    private BigDecimal couponTotalChargeQuantity = BigDecimal.ZERO;


    private BigDecimal couponTotalVolume = BigDecimal.ZERO;

    private BigDecimal couponTotalWeight = BigDecimal.ZERO;

    private BigDecimal couponTotalQuantity = BigDecimal.ZERO;

//    @ApiModelProperty(value = "使用该优惠的商品项列表")
//    private List<ProdConditionParam> prodConditionParamList;


//    public List<DiscountDetailedVO> getDiscountDetailedVOList() {
//        if (StringUtils.isNotBlank(this.discountDetailed)){
//            return JSONObject.parseArray(this.discountDetailed, DiscountDetailedVO.class);
//        }
//        return discountDetailedVOList;
//    }

    public void setDiscountDetailed(String discountDetailed) {
        this.discountDetailed = discountDetailed;
        this.discountDetailedVOList = JSONObject.parseArray(discountDetailed, DiscountDetailedVO.class);
    }
}
