package cn.iocoder.yudao.module.product.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ProductPriceExcelExportPushEvent;
import cn.iocoder.yudao.framework.excel.handler.WaterMarkHandler;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.ecw.service.unit.UnitService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.product.convert.product.ProductPriceConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.*;
import cn.iocoder.yudao.module.product.dto.ProductPriceExcelData;
import cn.iocoder.yudao.module.product.enums.PriceTypeEnum;
import cn.iocoder.yudao.module.product.service.product.*;
import cn.iocoder.yudao.module.product.vo.productPrice.ProductPriceExcelVO;
import cn.iocoder.yudao.module.product.vo.productPrice.ProductPricePageReqVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;


/**
 * 自编号订单excel导出监听
 *
 * @author zhengYi
 */
@Component("ProductPriceExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ProductPriceExcelExportListener {

    private UeProperties ueProperties;
    private FileService fileService;
    @Resource
    private ProductPriceService productPriceService;
    @Resource
    private CurrencyService currencyService;
    @Resource
    private UnitService unitService;

    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ProductPriceExcelExportPushEvent.class)
    public void productExcelExportPushEvent(ProductPriceExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                int pageSize = 10000;
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/productPrice/excel/");
                File fileDir = new File(dir);
                if (!fileDir.exists()) {
                    fileDir.mkdirs();
                }

                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("product_price.xlsx");

                ProductPricePageReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), ProductPricePageReqVO.class);
                productPriceService.initDictParams(exportReqVO);
                long total = productPriceService.getExcelListCount(exportReqVO);
                int pageNumber = (int) Math.ceil((double) total / (double) pageSize);

                String sheetName = "路线价格第{0}页";

                List<CurrencyDO> currencyList = currencyService.getCurrencyList();
                Map<Integer, CurrencyDO> currencyMap = currencyList.stream()
                        .collect(Collectors.toMap(CurrencyDO::getId, v -> v));

                List<UnitDO> unitList = unitService.getUnitList();
                Map<Integer, UnitDO> unitMap = unitList.stream()
                        .collect(Collectors.toMap(UnitDO::getId, v -> v));

                String path = dir + fileName;
                //分页条数看情况
                // 去调用写入,根据数据库分页的总的页数来
                OutputStream out = new FileOutputStream(path);
                ExcelWriter excelWriter = EasyExcel.write(out, ProductPriceExcelVO.class).excelType(ExcelTypeEnum.XLSX).build();
                Long maxId = null;
                for (int i = 1; i <= pageNumber; i++) {
                    WriteSheet writeSheet = EasyExcel.writerSheet(MessageFormat.format(sheetName, i)).build();
                    //先定义一个空集合每次循环使他变成null减少内存的占用
                    int start = (i - 1) * pageSize;
                    int end = i * pageSize;
                    List<ProductPriceExcelData> list =
                            productPriceService.getExcelList(pageSize, maxId, exportReqVO);
                    log.error("路线价格数据{}-{}处理成功", start, end);
                    if (CollectionUtil.isEmpty(list)) continue;
                    maxId = list.get(list.size() - 1).getId();

                    for (ProductPriceExcelData item : list) {

                        String transportPrice = "";
                        String clearancePrice = "";

                        if (item.getStepPrice() == 0) {
                            transportPrice = getPrice(item.getTransportPrice(),
                                    item.getTransportPriceUnit().intValue(),
                                    item.getTransportVolumeUnit().intValue(), currencyMap, unitMap);
                            clearancePrice = getPrice(item.getClearancePrice(),
                                    item.getClearancePriceUnit().intValue(),
                                    item.getClearanceVolumeUnit().intValue(), currencyMap, unitMap);

                            String allPrice = getPrice(item.getAllPrice(), item.getAllPriceUnit().intValue(),
                                    item.getAllVolumeUnit().intValue(), currencyMap, unitMap);

                            if (item.getPriceType() == 1) {
                                transportPrice = allPrice;
                                clearancePrice = "";
                            }

                            List<ProductPriceSpecialDO> specialList = item.getSpecialList();
                            if (CollectionUtil.isNotEmpty(specialList)) {
                                for (ProductPriceSpecialDO specialDO : specialList) {
                                    String key = specialDO.getSpecialDictType();
                                    String specialTransportPrice = getPrice(specialDO.getTransportPrice(),
                                            specialDO.getTransportPriceUnit().intValue(),
                                            specialDO.getTransportVolumeUnit().intValue(), currencyMap, unitMap);

                                    String specialClearancePrice = getPrice(specialDO.getClearancePrice(),
                                            specialDO.getClearancePriceUnit().intValue(),
                                            specialDO.getClearanceVolumeUnit().intValue(), currencyMap, unitMap);

                                    String specialAllPrice = getPrice(specialDO.getAllPrice(), specialDO.getAllPriceUnit().intValue(),
                                            specialDO.getAllVolumeUnit().intValue(), currencyMap, unitMap);
                                    if (item.getPriceType() == 1) {
                                        specialTransportPrice = specialAllPrice;
                                        specialClearancePrice = "";
                                    }
                                    if (key.equals("licensed_price")) {
                                        item.setYpTransportPrice(specialTransportPrice);
                                        item.setYpClearancePrice(specialClearancePrice);
                                    } else if (key.equals("neutral_brand_prices")) {
                                        item.setZxTransportPrice(specialTransportPrice);
                                        item.setZxClearancePrice(specialClearancePrice);
                                    } else if (key.equals("liquid_price")) {
                                        item.setYtTransportPrice(specialTransportPrice);
                                        item.setYtClearancePrice(specialClearancePrice);
                                    } else if (key.equals("live_price")) {
                                        item.setDdTransportPrice(specialTransportPrice);
                                        item.setDdClearancePrice(specialClearancePrice);
                                    }
                                }
                            }

                        } else if (item.getStepPrice() == 1) {
                            //阶梯价格
                            List<ProductPriceStepDO> stepList = item.getStepList();

                            if (CollectionUtil.isNotEmpty(stepList)) {
                                Map<Integer, List<ProductPriceStepDO>> stepTypeMap = stepList.stream()
                                        .collect(Collectors.groupingBy(ProductPriceStepDO::getPriceType));
                                List<ProductPriceStepDO> yfStepList = stepTypeMap.get(PriceTypeEnum.FREIGHT.getPriceType());
                                List<ProductPriceStepDO> qgfStepList = stepTypeMap.get(PriceTypeEnum.CLEARANCE.getPriceType());
                                List<ProductPriceStepDO> qbjStepList = stepTypeMap.get(PriceTypeEnum.FULL_PRICE.getPriceType());

                                StringBuilder transportPriceSb = new StringBuilder();
                                StringBuilder clearancePriceSb = new StringBuilder();

                                StringBuilder ypyfSb = new StringBuilder();
                                StringBuilder ypqgfSb = new StringBuilder();
                                StringBuilder zxyfSb = new StringBuilder();
                                StringBuilder zxqgfSb = new StringBuilder();
                                StringBuilder ytyfSb = new StringBuilder();
                                StringBuilder ytqgfSb = new StringBuilder();
                                StringBuilder ddyfSb = new StringBuilder();
                                StringBuilder ddqgfSb = new StringBuilder();
                                if (item.getPriceType() == 0) {
                                    if (CollectionUtil.isNotEmpty(yfStepList)) {
                                        yfStepList.sort(Comparator.comparing(ProductPriceStepDO::getRankNum));
                                        for (int j = 1; j <= yfStepList.size(); j++) {
                                            ProductPriceStepDO stepDO = yfStepList.get(j - 1);
                                            List<ProductPriceStepSpecialDO> stepSpecialDOS = stepDO.getSpecialList();
                                            if (CollectionUtil.isNotEmpty(stepSpecialDOS)) {
                                                Map<String, ProductPriceStepSpecialDO> stepSpecialDOMap = stepSpecialDOS.stream()
                                                        .collect(Collectors.toMap(ProductPriceStepSpecialDO::getSpecialDictType, v -> v, (v1, v2) -> v2));
                                                //有牌
                                                ProductPriceStepSpecialDO yp = stepSpecialDOMap.get("licensed_price");
                                                //中性
                                                ProductPriceStepSpecialDO zx = stepSpecialDOMap.get("neutral_brand_prices");
                                                //液体
                                                ProductPriceStepSpecialDO yt = stepSpecialDOMap.get("liquid_price");
                                                //带电
                                                ProductPriceStepSpecialDO dd = stepSpecialDOMap.get("live_price");
                                                String stepInfo = "第" + j + "阶梯：  ";

                                                addStepSpecialPrice(yp, currencyMap, unitMap, ypyfSb, stepInfo, PriceTypeEnum.FREIGHT);
                                                addStepSpecialPrice(zx, currencyMap, unitMap, zxyfSb, stepInfo, PriceTypeEnum.FREIGHT);
                                                addStepSpecialPrice(yt, currencyMap, unitMap, ytyfSb, stepInfo, PriceTypeEnum.FREIGHT);
                                                addStepSpecialPrice(dd, currencyMap, unitMap, ddyfSb, stepInfo, PriceTypeEnum.FREIGHT);
                                                if (i != yfStepList.size()) {
                                                    ypyfSb.append((char) 10);
                                                    zxyfSb.append((char) 10);
                                                    ytyfSb.append((char) 10);
                                                    ddyfSb.append((char) 10);
                                                }
                                            }

                                            String stepTransportPrice = getPrice(stepDO.getTransportPrice(),
                                                    stepDO.getTransportPriceUnit().intValue(),
                                                    stepDO.getTransportVolumeUnit().intValue(), currencyMap, unitMap);

                                            Long weightUnit = stepDO.getWeightUnit();
                                            String unit = "";
                                            if (weightUnit != null) {
                                                UnitDO unitDO = unitMap.get(weightUnit.intValue());
                                                unit = Objects.nonNull(unitDO) ? unitDO.getTitleZh() : "空";
                                            }
                                            BigDecimal startNum = stepDO.getStartNum();
                                            BigDecimal endNum = stepDO.getEndNum();
                                            transportPriceSb.append("第").append(j).append("阶梯：").append(startNum)
                                                    .append("-").append(endNum).append(unit).append("  ").append(stepTransportPrice);
                                            if (i != stepList.size()) {
                                                transportPriceSb.append((char) 10);
                                            }
                                        }
                                    }

                                    if (CollectionUtil.isNotEmpty(qgfStepList)) {
                                        qgfStepList.sort(Comparator.comparing(ProductPriceStepDO::getRankNum));
                                        for (int j = 1; j <= qgfStepList.size(); j++) {
                                            ProductPriceStepDO stepDO = qgfStepList.get(j - 1);
                                            List<ProductPriceStepSpecialDO> stepSpecialDOS = stepDO.getSpecialList();
                                            if (CollectionUtil.isNotEmpty(stepSpecialDOS)) {
                                                Map<String, ProductPriceStepSpecialDO> stepSpecialDOMap = stepSpecialDOS.stream()
                                                        .collect(Collectors.toMap(ProductPriceStepSpecialDO::getSpecialDictType, v -> v, (v1, v2) -> v2));
                                                //有牌
                                                ProductPriceStepSpecialDO yp = stepSpecialDOMap.get("licensed_price");
                                                //中性
                                                ProductPriceStepSpecialDO zx = stepSpecialDOMap.get("neutral_brand_prices");
                                                //液体
                                                ProductPriceStepSpecialDO yt = stepSpecialDOMap.get("liquid_price");
                                                //带电
                                                ProductPriceStepSpecialDO dd = stepSpecialDOMap.get("live_price");
                                                String stepInfo = "第" + j + "阶梯：  ";

                                                addStepSpecialPrice(yp, currencyMap, unitMap, ypqgfSb, stepInfo, PriceTypeEnum.CLEARANCE);
                                                addStepSpecialPrice(zx, currencyMap, unitMap, zxqgfSb, stepInfo, PriceTypeEnum.CLEARANCE);
                                                addStepSpecialPrice(yt, currencyMap, unitMap, ytqgfSb, stepInfo, PriceTypeEnum.CLEARANCE);
                                                addStepSpecialPrice(dd, currencyMap, unitMap, ddqgfSb, stepInfo, PriceTypeEnum.CLEARANCE);
                                                if (i != qgfStepList.size()) {
                                                    ypqgfSb.append((char) 10);
                                                    zxqgfSb.append((char) 10);
                                                    ytqgfSb.append((char) 10);
                                                    ddqgfSb.append((char) 10);
                                                }
                                            }

                                            String stepClearancePrice = getPrice(stepDO.getClearancePrice(),
                                                    stepDO.getClearancePriceUnit().intValue(),
                                                    stepDO.getClearanceVolumeUnit().intValue(), currencyMap, unitMap);
//
                                            Long weightUnit = stepDO.getWeightUnit();
                                            String unit = "";
                                            if (weightUnit != null) {
                                                UnitDO unitDO = unitMap.get(weightUnit.intValue());
                                                unit = Objects.nonNull(unitDO) ? unitDO.getTitleZh() : "空";
                                            }
                                            BigDecimal startNum = stepDO.getStartNum();
                                            BigDecimal endNum = stepDO.getEndNum();
                                            clearancePriceSb.append("第").append(j).append("阶梯：").append(startNum)
                                                    .append("-").append(endNum).append(unit).append("  ").append(stepClearancePrice);
                                            if (i != stepList.size()) {
                                                clearancePriceSb.append((char) 10);
                                            }
                                        }
                                    }
                                }
                                if (item.getPriceType() == 1) {
                                    if (CollectionUtil.isNotEmpty(qbjStepList) && item.getPriceType() == 1) {
                                        qbjStepList.sort(Comparator.comparing(ProductPriceStepDO::getRankNum));
                                        for (int j = 1; j <= qbjStepList.size(); j++) {
                                            ProductPriceStepDO stepDO = qbjStepList.get(j - 1);
                                            List<ProductPriceStepSpecialDO> stepSpecialDOS = stepDO.getSpecialList();
                                            if (CollectionUtil.isNotEmpty(stepSpecialDOS)) {
                                                Map<String, ProductPriceStepSpecialDO> stepSpecialDOMap = stepSpecialDOS.stream()
                                                        .collect(Collectors.toMap(ProductPriceStepSpecialDO::getSpecialDictType, v -> v, (v1, v2) -> v2));
                                                //有牌
                                                ProductPriceStepSpecialDO yp = stepSpecialDOMap.get("licensed_price");
                                                //中性
                                                ProductPriceStepSpecialDO zx = stepSpecialDOMap.get("neutral_brand_prices");
                                                //液体
                                                ProductPriceStepSpecialDO yt = stepSpecialDOMap.get("liquid_price");
                                                //带电
                                                ProductPriceStepSpecialDO dd = stepSpecialDOMap.get("live_price");
                                                String stepInfo = "第" + j + "阶梯：  ";

                                                addStepSpecialPrice(yp, currencyMap, unitMap, ypyfSb, stepInfo, PriceTypeEnum.FULL_PRICE);
                                                addStepSpecialPrice(zx, currencyMap, unitMap, zxyfSb, stepInfo, PriceTypeEnum.FULL_PRICE);
                                                addStepSpecialPrice(yt, currencyMap, unitMap, ytyfSb, stepInfo, PriceTypeEnum.FULL_PRICE);
                                                addStepSpecialPrice(dd, currencyMap, unitMap, ddyfSb, stepInfo, PriceTypeEnum.FULL_PRICE);
                                                if (i != qbjStepList.size()) {
                                                    ypyfSb.append((char) 10);
                                                    zxyfSb.append((char) 10);
                                                    ytyfSb.append((char) 10);
                                                    ddyfSb.append((char) 10);
                                                }
                                            }

//
                                            String stepTransportPrice = getPrice(stepDO.getAllPrice(), stepDO.getAllPriceUnit().intValue(),
                                                    stepDO.getAllVolumeUnit().intValue(), currencyMap, unitMap);
//
                                            Long weightUnit = stepDO.getWeightUnit();
                                            String unit = "";
                                            if (weightUnit != null) {
                                                UnitDO unitDO = unitMap.get(weightUnit.intValue());
                                                unit = Objects.nonNull(unitDO) ? unitDO.getTitleZh() : "空";
                                            }
                                            BigDecimal startNum = stepDO.getStartNum();
                                            BigDecimal endNum = stepDO.getEndNum();
                                            transportPriceSb.append("第").append(j).append("阶梯：").append(startNum)
                                                    .append("-").append(endNum).append(unit).append("  ").append(stepTransportPrice);
                                            if (i != stepList.size()) {
                                                transportPriceSb.append((char) 10);
                                            }
                                        }
                                    }
                                }
                                transportPrice = transportPriceSb.toString();
                                clearancePrice = clearancePriceSb.toString();

                                item.setYpTransportPrice(ypyfSb.toString());
                                item.setYpClearancePrice(ypqgfSb.toString());
                                item.setZxTransportPrice(zxyfSb.toString());
                                item.setZxClearancePrice(zxqgfSb.toString());
                                item.setYtTransportPrice(ytyfSb.toString());
                                item.setYtClearancePrice(ytqgfSb.toString());
                                item.setDdTransportPrice(ddyfSb.toString());
                                item.setDdClearancePrice(ddqgfSb.toString());
                            }
                        }

                        item.setMrTransportPrice(transportPrice);
                        item.setMrClearancePrice(clearancePrice);


                        statusName(item);
                        if (item.getNeedPay() == 0) {
                            item.setPayType("到付");
                        } else if (item.getNeedPay() == 1) {
                            item.setPayType("预付");
                        }
                        String date = DateUtil.format(item.getUpdateTime(), "yyyy-MM-dd HH:mm:ss");
                        item.setLastTime(date);

                        Date startDate = item.getValidateStartDate();
                        Date endDate = item.getValidateEndDate();
                        StringBuilder expireSb = new StringBuilder();
                        if (startDate != null) {
                            String startDateStr = DateUtil.format(startDate, "yyyy-MM-dd");
                            expireSb.append(startDateStr);
                        }
                        expireSb.append("~");
                        if (endDate != null) {
                            String endDateStr = DateUtil.format(endDate, "yyyy-MM-dd");
                            expireSb.append(endDateStr);
                        }
                        item.setExpireTime(expireSb.toString());
                    }
                    List<ProductPriceExcelVO> datas = ProductPriceConvert.INSTANCE.convertList02(list);
                    excelWriter.write(datas, writeSheet);
                    list.forEach(m -> {
                        if (m.getSpecialList() != null) {
                            m.getSpecialList().clear();
                        }
                        if (m.getStepList() != null) {
                            m.getStepList().clear();
                        }
                    });
                    list.clear();
                    datas.clear();
                }
                excelWriter.finish();
                out.flush();
                // 获取到临时文件
                File file = new File(path);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                log.error(e + "");
                event.setResult(e + "");
            }
        } else {
            event.setResult("param fail");
        }

    }

    private void addStepSpecialPrice(ProductPriceStepSpecialDO stepSpecialDO, Map<Integer, CurrencyDO> currencyMap,
                                     Map<Integer, UnitDO> unitMap, StringBuilder sb, String stepInfo, PriceTypeEnum priceTypeEnum) {
        sb.append(stepInfo);
        if (stepSpecialDO != null) {
            if (priceTypeEnum == PriceTypeEnum.FREIGHT) {
                String specialTransportPrice = getPrice(stepSpecialDO.getTransportPrice(),
                        stepSpecialDO.getTransportPriceUnit().intValue(),
                        stepSpecialDO.getTransportVolumeUnit().intValue(), currencyMap, unitMap);
                sb.append(specialTransportPrice);
            } else if (priceTypeEnum == PriceTypeEnum.CLEARANCE) {
                String specialClearancePrice = getPrice(stepSpecialDO.getClearancePrice(),
                        stepSpecialDO.getClearancePriceUnit().intValue(),
                        stepSpecialDO.getClearanceVolumeUnit().intValue(), currencyMap, unitMap);
                sb.append(specialClearancePrice);
            } else if (priceTypeEnum == PriceTypeEnum.FULL_PRICE) {
                String specialAllPrice = getPrice(stepSpecialDO.getAllPrice(), stepSpecialDO.getAllPriceUnit().intValue(),
                        stepSpecialDO.getAllVolumeUnit().intValue(), currencyMap, unitMap);
                sb.append(specialAllPrice);
            }
        }
    }

    private String getPrice(BigDecimal price, int priceUnit, int volumeUnit, Map<Integer, CurrencyDO> currencyMap, Map<Integer, UnitDO> unitMap) {
        CurrencyDO currencyDO = currencyMap.get(priceUnit);
        UnitDO unitDO = unitMap.get(volumeUnit);

        return (currencyDO != null ? currencyDO.getFuhao() : "空") + price.toString() + (currencyDO != null ? currencyDO.getTitleZh() : "空") + "/" +
                (unitDO != null ? unitDO.getTitleZh() : "空");
    }

    private void statusName(ProductPriceExcelData item) {
        Date startDate = item.getValidateStartDate();
        Date endDate = item.getValidateEndDate();
        boolean notSetPrice = false;
        long now = new Date().getTime();
        if (startDate != null && startDate.getTime() > now) {
            notSetPrice = true;
        }
        if (endDate != null && endDate.getTime() < now) {
            notSetPrice = true;
        }
        String setPriceText = notSetPrice ? "（未设置价格）" : "";
        if (item.getBlacklist() == 1) {
            item.setStatusName("黑名单" + setPriceText);
            return;
        }
        if (item.getAuditStatus() == 0) {
            item.setStatusName("已审核" + setPriceText);
            return;
        }
        item.setStatusName("已下架" + setPriceText);
    }
}
