package cn.iocoder.yudao.module.product.service.coupon;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentCreateReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentPageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponContentDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 优惠券站内信息关联信息 Service 接口
 *
 * @author 系统管理员
 */
public interface CouponContentService extends IService<CouponContentDO> {

    /**
     * 创建优惠券站内信息关联信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCouponContent(@Valid CouponContentCreateReqVO createReqVO);

    /**
     * 更新优惠券站内信息关联信息
     *
     * @param updateReqVO 更新信息
     */
    void updateCouponContent(@Valid CouponContentUpdateReqVO updateReqVO);

    /**
     * 删除优惠券站内信息关联信息
     *
     * @param id 编号
     */
    void deleteCouponContent(Long id);

    /**
     * 获得优惠券站内信息关联信息
     *
     * @param id 编号
     * @return 优惠券站内信息关联信息
     */
    CouponContentDO getCouponContent(Long id);

    /**
     * 获得优惠券站内信息关联信息列表
     *
     * @param ids 编号
     * @return 优惠券站内信息关联信息列表
     */
    List<CouponContentDO> getCouponContentList(Collection<Long> ids);

    /**
     * 获得优惠券站内信息关联信息分页
     *
     * @param pageReqVO 分页查询
     * @return 优惠券站内信息关联信息分页
     */
    PageResult<CouponContentDO> getCouponContentPage(CouponContentPageReqVO pageReqVO);

    /**
     * 获得优惠券站内信息关联信息列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 优惠券站内信息关联信息列表
     */
    List<CouponContentDO> getCouponContentList(CouponContentExportReqVO exportReqVO);

}
