package cn.iocoder.yudao.module.product.service.coupon;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.product.vo.coupon.customer.*;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponCustomerDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 优惠券客户关联信息 Service 接口
 *
 * @author 系统管理员
 */
public interface CouponCustomerService extends IService<CouponCustomerDO> {

    /**
     * 创建优惠券客户关联信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCouponCustomer(@Valid CouponCustomerCreateReqVO createReqVO);

    /**
     * 更新优惠券客户关联信息
     *
     * @param updateReqVO 更新信息
     */
    void updateCouponCustomer(@Valid CouponCustomerUpdateReqVO updateReqVO);

    /**
     * 删除优惠券客户关联信息
     *
     * @param id 编号
     */
    void deleteCouponCustomer(Long id);

    /**
     * 获得优惠券客户关联信息
     *
     * @param id 编号
     * @return 优惠券客户关联信息
     */
    CouponCustomerDO getCouponCustomer(Long id);

    /**
     * 获得优惠券客户关联信息列表
     *
     * @param ids 编号
     * @return 优惠券客户关联信息列表
     */
    List<CouponCustomerDO> getCouponCustomerList(Collection<Long> ids);

    /**
     * 获得优惠券客户关联信息分页
     *
     * @param pageReqVO 分页查询
     * @return 优惠券客户关联信息分页
     */
    PageResult<CouponCustomerDO> getCouponCustomerPage(CouponCustomerPageReqVO pageReqVO);

    /**
     * 获得优惠券客户关联信息列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 优惠券客户关联信息列表
     */
    List<CouponCustomerDO> getCouponCustomerList(CouponCustomerExportReqVO exportReqVO);

    /**
     * @param couponCustomerIds 用户优惠劵ids
     * @param customerId 用户ID
     * @return
     */
    List<CouponCustomerInfoVO> listInfo(String couponCustomerIds, Long customerId);

    PageResult<CouponCustomerRespVO> userCouponPage(PageVO page, CouponCustomerPageReqVO reqVO);

    List<Long> getCustomerIdsByCouponId(Long couponId);
}
