package cn.iocoder.yudao.module.product.service.coupon;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLineCreateReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLineExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLinePageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLineUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponLineDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 优惠券线路关联信息 Service 接口
 *
 * @author 系统管理员
 */
public interface CouponLineService extends IService<CouponLineDO> {

    /**
     * 创建优惠券线路关联信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCouponLine(@Valid CouponLineCreateReqVO createReqVO);

    /**
     * 更新优惠券线路关联信息
     *
     * @param updateReqVO 更新信息
     */
    void updateCouponLine(@Valid CouponLineUpdateReqVO updateReqVO);

    /**
     * 删除优惠券线路关联信息
     *
     * @param id 编号
     */
    void deleteCouponLine(Long id);

    /**
     * 获得优惠券线路关联信息
     *
     * @param id 编号
     * @return 优惠券线路关联信息
     */
    CouponLineDO getCouponLine(Long id);

    /**
     * 获得优惠券线路关联信息列表
     *
     * @param ids 编号
     * @return 优惠券线路关联信息列表
     */
    List<CouponLineDO> getCouponLineList(Collection<Long> ids);

    /**
     * 获得优惠券线路关联信息分页
     *
     * @param pageReqVO 分页查询
     * @return 优惠券线路关联信息分页
     */
    PageResult<CouponLineDO> getCouponLinePage(CouponLinePageReqVO pageReqVO);

    /**
     * 获得优惠券线路关联信息列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 优惠券线路关联信息列表
     */
    List<CouponLineDO> getCouponLineList(CouponLineExportReqVO exportReqVO);

}
