package cn.iocoder.yudao.module.product.service.coupon;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdCreateReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdPageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponProdDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 优惠券商品关联信息 Service 接口
 *
 * @author 系统管理员
 */
public interface CouponProdService extends IService<CouponProdDO> {

    /**
     * 创建优惠券商品关联信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCouponProd(@Valid CouponProdCreateReqVO createReqVO);

    /**
     * 更新优惠券商品关联信息
     *
     * @param updateReqVO 更新信息
     */
    void updateCouponProd(@Valid CouponProdUpdateReqVO updateReqVO);

    /**
     * 删除优惠券商品关联信息
     *
     * @param id 编号
     */
    void deleteCouponProd(Long id);

    /**
     * 获得优惠券商品关联信息
     *
     * @param id 编号
     * @return 优惠券商品关联信息
     */
    CouponProdDO getCouponProd(Long id);

    /**
     * 获得优惠券商品关联信息列表
     *
     * @param ids 编号
     * @return 优惠券商品关联信息列表
     */
    List<CouponProdDO> getCouponProdList(Collection<Long> ids);

    /**
     * 获得优惠券商品关联信息分页
     *
     * @param pageReqVO 分页查询
     * @return 优惠券商品关联信息分页
     */
    PageResult<CouponProdDO> getCouponProdPage(CouponProdPageReqVO pageReqVO);

    /**
     * 获得优惠券商品关联信息列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 优惠券商品关联信息列表
     */
    List<CouponProdDO> getCouponProdList(CouponProdExportReqVO exportReqVO);

}
