package cn.iocoder.yudao.module.product.service.coupon;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.*;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponUseRecordDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 优惠券使用记录 Service 接口
 *
 * @author 系统管理员
 */
public interface CouponUseRecordService extends IService<CouponUseRecordDO> {

    /**
     * 创建优惠券使用记录
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCouponUseRecord(@Valid CouponUseRecordCreateReqVO createReqVO);


    /**
     * 批量使用优惠劵
     *
     * @param orderNumber 订单流水号
     * @param userId      用户ID
     * @param customerId  客户ID
     * @param couponIds   优惠劵ID列表
     */
    void useCoupon(String orderNumber, Long userId, Long customerId, List<Long> couponIds);

    /**
     * 更新优惠券使用记录
     *
     * @param updateReqVO 更新信息
     */
    void updateCouponUseRecord(@Valid CouponUseRecordUpdateReqVO updateReqVO);

    /**
     * 删除优惠券使用记录
     *
     * @param id 编号
     */
    void deleteCouponUseRecord(Long id);

    /**
     * 获得优惠券使用记录
     *
     * @param id 编号
     * @return 优惠券使用记录
     */
    CouponUseRecordDO getCouponUseRecord(Long id);

    /**
     * 获得优惠券使用记录列表
     *
     * @param ids 编号
     * @return 优惠券使用记录列表
     */
    List<CouponUseRecordDO> getCouponUseRecordList(Collection<Long> ids);

    /**
     * 获得优惠券使用记录分页
     *
     * @param pageReqVO 分页查询
     * @return 优惠券使用记录分页
     */
    PageResult<CouponUseRecordDO> getCouponUseRecordPage(CouponUseRecordPageReqVO pageReqVO);

    /**
     * 获得优惠券使用记录列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 优惠券使用记录列表
     */
    List<CouponUseRecordDO> getCouponUseRecordList(CouponUseRecordExportReqVO exportReqVO);

    CouponUseRecordDO info(Long id);

    PageResult<CouponUseRecordRespVO> couponUseRecordPage(PageVO page, CouponUseRecordPageReqVO reqVO);
}
