package cn.iocoder.yudao.module.product.service.coupon.impl;

import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentCreateReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentPageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.content.CouponContentUpdateReqVO;
import cn.iocoder.yudao.module.product.service.coupon.CouponContentService;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.CouponContentMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponContentDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.product.convert.coupon.CouponContentConvert;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.COUPON_CONTENT_NOT_EXISTS;

/**
 * 优惠券站内信息关联信息 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CouponContentServiceImpl extends AbstractService<CouponContentMapper, CouponContentDO> implements CouponContentService {

    @Autowired
    private CouponContentMapper couponContentMapper;

    @Override
    public Long createCouponContent(CouponContentCreateReqVO createReqVO) {
        // 插入
        CouponContentDO couponContent = CouponContentConvert.INSTANCE.convert(createReqVO);
        couponContentMapper.insert(couponContent);
        // 返回
        return couponContent.getCouponContentId();
    }

    @Override
    public void updateCouponContent(CouponContentUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCouponContentExists(updateReqVO.getCouponContentId());
        // 更新
        CouponContentDO updateObj = CouponContentConvert.INSTANCE.convert(updateReqVO);
        couponContentMapper.updateById(updateObj);
    }

    @Override
    public void deleteCouponContent(Long id) {
        // 校验存在
        this.validateCouponContentExists(id);
        // 删除
        couponContentMapper.deleteById(id);
    }

    private void validateCouponContentExists(Long id) {
        if (couponContentMapper.selectById(id) == null) {
            throw exception(COUPON_CONTENT_NOT_EXISTS);
        }
    }

    @Override
    public CouponContentDO getCouponContent(Long id) {
        return couponContentMapper.selectById(id);
    }

    @Override
    public List<CouponContentDO> getCouponContentList(Collection<Long> ids) {
        return couponContentMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CouponContentDO> getCouponContentPage(CouponContentPageReqVO pageReqVO) {
        return couponContentMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CouponContentDO> getCouponContentList(CouponContentExportReqVO exportReqVO) {
        return couponContentMapper.selectList(exportReqVO);
    }

}
