package cn.iocoder.yudao.module.product.service.coupon.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponLineDO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponProdDO;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.CouponProdMapper;
import cn.iocoder.yudao.module.product.service.coupon.CouponLineService;
import cn.iocoder.yudao.module.product.service.coupon.CouponProdService;
import cn.iocoder.yudao.module.product.vo.coupon.customer.*;
import cn.iocoder.yudao.module.product.service.coupon.CouponCustomerService;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.CouponCustomerMapper;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordRespVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;

import java.util.*;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponCustomerDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.product.convert.coupon.CouponCustomerConvert;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.COUPON_CUSTOMER_NOT_EXISTS;

/**
 * 优惠券客户关联信息 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CouponCustomerServiceImpl extends AbstractService<CouponCustomerMapper, CouponCustomerDO> implements CouponCustomerService {

    @Autowired
    private CouponCustomerMapper couponCustomerMapper;
    @Autowired
    private CouponProdService couponProdService;
    @Autowired
    private CouponLineService couponLineService;

    @Override
    public Long createCouponCustomer(CouponCustomerCreateReqVO createReqVO) {
        // 插入
        CouponCustomerDO couponCustomer = CouponCustomerConvert.INSTANCE.convert(createReqVO);
        couponCustomerMapper.insert(couponCustomer);
        // 返回
        return couponCustomer.getCouponCustomerId();
    }

    @Override
    public void updateCouponCustomer(CouponCustomerUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCouponCustomerExists(updateReqVO.getCouponCustomerId());
        // 更新
        CouponCustomerDO updateObj = CouponCustomerConvert.INSTANCE.convert(updateReqVO);
        couponCustomerMapper.updateById(updateObj);
    }

    @Override
    public void deleteCouponCustomer(Long id) {
        // 校验存在
        this.validateCouponCustomerExists(id);
        // 删除
        couponCustomerMapper.deleteById(id);
    }

    private void validateCouponCustomerExists(Long id) {
        if (couponCustomerMapper.selectById(id) == null) {
            throw exception(COUPON_CUSTOMER_NOT_EXISTS);
        }
    }

    @Override
    public CouponCustomerDO getCouponCustomer(Long id) {
        return couponCustomerMapper.selectById(id);
    }

    @Override
    public List<CouponCustomerDO> getCouponCustomerList(Collection<Long> ids) {
        return couponCustomerMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CouponCustomerDO> getCouponCustomerPage(CouponCustomerPageReqVO pageReqVO) {
        return couponCustomerMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CouponCustomerDO> getCouponCustomerList(CouponCustomerExportReqVO exportReqVO) {
        return couponCustomerMapper.selectList(exportReqVO);
    }

    @Override
    public List<CouponCustomerInfoVO> listInfo(String couponCustomerIds, Long customerId) {
        List<CouponCustomerInfoVO> couponCustomerInfoVOList = couponCustomerMapper.listInfo(Arrays.asList(couponCustomerIds.split(StrUtil.COMMA)), customerId);
        couponCustomerInfoVOList = couponCustomerInfoVOList.stream().peek(vo -> {
            List<CouponProdDO> couponProdDOList = couponProdService.selectList(new LambdaQueryWrapper<CouponProdDO>()
                    .eq(CouponProdDO::getCouponId, vo.getCouponId()));
            if (CollectionUtil.isNotEmpty(couponProdDOList)){
                vo.setCouponProdList(couponProdDOList);
            }
            List<CouponLineDO> couponLineDOList = couponLineService.selectList(new LambdaQueryWrapper<CouponLineDO>()
                    .eq(CouponLineDO::getCouponId, vo.getCouponId()));
            if (CollectionUtil.isNotEmpty(couponLineDOList)){
                vo.setCouponLineList(couponLineDOList);
            }
        }).collect(Collectors.toList());
        return couponCustomerInfoVOList;
    }

    @Override
    public PageResult<CouponCustomerRespVO> userCouponPage(PageVO page, CouponCustomerPageReqVO reqVO) {
        reqVO.setLang(I18nMessage.getLang());
        page.setField("cc.coupon_customer_id");
        IPage<CouponCustomerRespVO> mpPage = MyBatisUtils.buildPage(page);
        couponCustomerMapper.couponUseRecordPage(mpPage, reqVO);
        return PageResult.of(mpPage);
    }


    @Override
    public List<Long> getCustomerIdsByCouponId(Long couponId) {
        return couponCustomerMapper.getCustomerIdsByCouponId(couponId);
    }
}
