package cn.iocoder.yudao.module.product.service.coupon.impl;

import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLineCreateReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLineExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLinePageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.line.CouponLineUpdateReqVO;
import cn.iocoder.yudao.module.product.service.coupon.CouponLineService;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.CouponLineMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponLineDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.product.convert.coupon.CouponLineConvert;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.COUPON_LINE_NOT_EXISTS;

/**
 * 优惠券线路关联信息 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CouponLineServiceImpl extends AbstractService<CouponLineMapper, CouponLineDO> implements CouponLineService {

    @Autowired
    private CouponLineMapper couponLineMapper;

    @Override
    public Long createCouponLine(CouponLineCreateReqVO createReqVO) {
        // 插入
        CouponLineDO couponLine = CouponLineConvert.INSTANCE.convert(createReqVO);
        couponLineMapper.insert(couponLine);
        // 返回
        return couponLine.getCouponLineId();
    }

    @Override
    public void updateCouponLine(CouponLineUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCouponLineExists(updateReqVO.getCouponLineId());
        // 更新
        CouponLineDO updateObj = CouponLineConvert.INSTANCE.convert(updateReqVO);
        couponLineMapper.updateById(updateObj);
    }

    @Override
    public void deleteCouponLine(Long id) {
        // 校验存在
        this.validateCouponLineExists(id);
        // 删除
        couponLineMapper.deleteById(id);
    }

    private void validateCouponLineExists(Long id) {
        if (couponLineMapper.selectById(id) == null) {
            throw exception(COUPON_LINE_NOT_EXISTS);
        }
    }

    @Override
    public CouponLineDO getCouponLine(Long id) {
        return couponLineMapper.selectById(id);
    }

    @Override
    public List<CouponLineDO> getCouponLineList(Collection<Long> ids) {
        return couponLineMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CouponLineDO> getCouponLinePage(CouponLinePageReqVO pageReqVO) {
        return couponLineMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CouponLineDO> getCouponLineList(CouponLineExportReqVO exportReqVO) {
        return couponLineMapper.selectList(exportReqVO);
    }

}
