package cn.iocoder.yudao.module.product.service.coupon.impl;

import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdCreateReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdPageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.prod.CouponProdUpdateReqVO;
import cn.iocoder.yudao.module.product.service.coupon.CouponProdService;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.CouponProdMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponProdDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.product.convert.coupon.CouponProdConvert;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.COUPON_PROD_NOT_EXISTS;

/**
 * 优惠券商品关联信息 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CouponProdServiceImpl extends AbstractService<CouponProdMapper, CouponProdDO> implements CouponProdService {

    @Autowired
    private CouponProdMapper couponProdMapper;

    @Override
    public Long createCouponProd(CouponProdCreateReqVO createReqVO) {
        // 插入
        CouponProdDO couponProd = CouponProdConvert.INSTANCE.convert(createReqVO);
        couponProdMapper.insert(couponProd);
        // 返回
        return couponProd.getCouponProdId();
    }

    @Override
    public void updateCouponProd(CouponProdUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCouponProdExists(updateReqVO.getCouponProdId());
        // 更新
        CouponProdDO updateObj = CouponProdConvert.INSTANCE.convert(updateReqVO);
        couponProdMapper.updateById(updateObj);
    }

    @Override
    public void deleteCouponProd(Long id) {
        // 校验存在
        this.validateCouponProdExists(id);
        // 删除
        couponProdMapper.deleteById(id);
    }

    private void validateCouponProdExists(Long id) {
        if (couponProdMapper.selectById(id) == null) {
            throw exception(COUPON_PROD_NOT_EXISTS);
        }
    }

    @Override
    public CouponProdDO getCouponProd(Long id) {
        return couponProdMapper.selectById(id);
    }

    @Override
    public List<CouponProdDO> getCouponProdList(Collection<Long> ids) {
        return couponProdMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CouponProdDO> getCouponProdPage(CouponProdPageReqVO pageReqVO) {
        return couponProdMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CouponProdDO> getCouponProdList(CouponProdExportReqVO exportReqVO) {
        return couponProdMapper.selectList(exportReqVO);
    }

}
