package cn.iocoder.yudao.module.product.service.coupon.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponCustomerDO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponDO;
import cn.iocoder.yudao.module.product.service.coupon.CouponCustomerService;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import cn.iocoder.yudao.module.product.vo.coupon.record.*;
import cn.iocoder.yudao.module.product.service.coupon.CouponUseRecordService;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.CouponUseRecordMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponUseRecordDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.product.convert.coupon.CouponUseRecordConvert;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;

/**
 * 优惠券使用记录 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CouponUseRecordServiceImpl extends AbstractService<CouponUseRecordMapper, CouponUseRecordDO> implements CouponUseRecordService {

    @Autowired
    private CouponUseRecordMapper couponUseRecordMapper;
    @Autowired
    private CouponCustomerService couponCustomerService;
    @Autowired
    private CouponService couponService;

    @Override
    public Long createCouponUseRecord(CouponUseRecordCreateReqVO createReqVO) {
        // 插入
        CouponUseRecordDO couponUseRecord = CouponUseRecordConvert.INSTANCE.convert(createReqVO);
        couponUseRecordMapper.insert(couponUseRecord);
        // 返回
        return couponUseRecord.getCouponUseRecordId();
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void useCoupon(String orderNumber, Long userId, Long customerId, List<Long> couponIds) {
        List<CouponCustomerDO> couponCustomerDOList = couponCustomerService.selectList(new LambdaQueryWrapper<CouponCustomerDO>()
                .in(CouponCustomerDO::getCouponId, couponIds)
                .or(w -> w.eq(CouponCustomerDO::getCustomerId, customerId))
                .or(w -> w.eq(CouponCustomerDO::getUserId,userId)));
        List<Long> customerCouponIds = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(couponCustomerDOList)){
            for (CouponCustomerDO couponCustomerDO : couponCustomerDOList) {
                if (couponCustomerDO.getStatus() == 0 || couponCustomerDO.getUserStartTime().compareTo(new Date()) > 0 || couponCustomerDO.getUserEndTime().compareTo(new Date()) < 0){
                    // 不能使用失效优惠劵
                    throw exception(NO_USE_EXPIRE_COUPON);
                }
                if (couponCustomerDO.getStatus() == 2 ){
                    // 优惠劵已被使用
                    throw exception(COUPON_HAS_BEEN_USE);
                }
                // 如果是未被使用过的，则修改为已使用
                couponCustomerDO.setStatus(2);
                customerCouponIds.add(couponCustomerDO.getCouponId());
                CouponUseRecordDO recordDO = new CouponUseRecordDO();
                recordDO.setStatus(1); // 是否直接标记已使用，(暂时先冻结，以防有修改回退优惠劵)
                recordDO.setOrderNumber(orderNumber);
                recordDO.setCustomerId(customerId);
                recordDO.setUserId(userId);
                recordDO.setCouponCustomerId(couponCustomerDO.getCouponCustomerId());
                recordDO.setUseTime(new Date());
                couponUseRecordMapper.insert(recordDO);
            }
        }
        List<Long> notUseCouponIds;
        if (CollectionUtil.isNotEmpty(customerCouponIds)) {
            notUseCouponIds = couponIds.stream().filter(cid -> !CollectionUtil.contains(customerCouponIds, cid)).collect(Collectors.toList());
        }else {
            notUseCouponIds = new ArrayList<>(couponIds);
        }
        couponCustomerService.updateBatch(couponCustomerDOList);
        for (Long notUseCouponId : notUseCouponIds) {
            CouponDO couponDO = couponService.getCoupon(notUseCouponId);
            if (couponDO.getStatus() != 1 || couponDO.getStartTime().compareTo(new Date()) > 0 || couponDO.getEndTime().compareTo(DateUtil.beginOfDay(new Date())) < 0){
                // 不能使用失效优惠劵
                throw exception(NO_USE_EXPIRE_COUPON);
            }
            // 为用户领取一个优惠记录
            CouponCustomerDO couponCustomerDO = new CouponCustomerDO();
            couponCustomerDO.setUserId(userId);
            couponCustomerDO.setCustomerId(customerId);
            couponCustomerDO.setCouponId(couponDO.getCouponId());
            couponCustomerDO.setReceiveTime(new Date());
            couponCustomerDO.setStatus(2);
            couponCustomerDO.setUserStartTime(couponDO.getStartTime());
            couponCustomerDO.setUserEndTime(couponDO.getEndTime());
            couponCustomerService.save(couponCustomerDO);
            // 创建没有绑定用户的优惠活动参与记录
            CouponUseRecordDO recordDO = new CouponUseRecordDO();
            recordDO.setStatus(1); // TODO 是否直接标记已使用，(暂时先冻结，以防有修改回退优惠劵)
            recordDO.setOrderNumber(orderNumber);
            recordDO.setCustomerId(customerId);
            recordDO.setUserId(userId);
            recordDO.setCouponUseRecordId(couponCustomerDO.getCouponCustomerId());
        }

    }

    @Override
    public void updateCouponUseRecord(CouponUseRecordUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCouponUseRecordExists(updateReqVO.getCouponUseRecordId());
        // 更新
        CouponUseRecordDO updateObj = CouponUseRecordConvert.INSTANCE.convert(updateReqVO);
        couponUseRecordMapper.updateById(updateObj);
    }

    @Override
    public void deleteCouponUseRecord(Long id) {
        // 校验存在
        this.validateCouponUseRecordExists(id);
        // 删除
        couponUseRecordMapper.deleteById(id);
    }

    private void validateCouponUseRecordExists(Long id) {
        if (couponUseRecordMapper.selectById(id) == null) {
            throw exception(COUPON_USE_RECORD_NOT_EXISTS);
        }
    }

    @Override
    public CouponUseRecordDO getCouponUseRecord(Long id) {
        return couponUseRecordMapper.selectById(id);
    }

    @Override
    public List<CouponUseRecordDO> getCouponUseRecordList(Collection<Long> ids) {
        return couponUseRecordMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CouponUseRecordDO> getCouponUseRecordPage(CouponUseRecordPageReqVO pageReqVO) {
        return couponUseRecordMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CouponUseRecordDO> getCouponUseRecordList(CouponUseRecordExportReqVO exportReqVO) {
        return couponUseRecordMapper.selectList(exportReqVO);
    }

    @Override
    public CouponUseRecordDO info(Long id) {
        return couponUseRecordMapper.info(id);
    }

    @Override
    public PageResult<CouponUseRecordRespVO> couponUseRecordPage(PageVO page, CouponUseRecordPageReqVO reqVO) {
        reqVO.setLang(I18nMessage.getLang());
        page.setField("r.coupon_use_record_id");
        IPage<CouponUseRecordRespVO> mpPage = MyBatisUtils.buildPage(page);
        couponUseRecordMapper.couponUseRecordPage(mpPage, reqVO);
        return PageResult.of(mpPage);
    }
}
