package cn.iocoder.yudao.module.product.service.product;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrCreateReqVO;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrExportReqVO;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrPageReqVO;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;

/**
 * 产品属性 Service 接口
 *
 * @author wlh
 */
public interface ProductAttrService {

    /**
     * 创建产品属性
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProductAttr(@Valid ProductAttrCreateReqVO createReqVO);

    /**
     * 更新产品属性
     *
     * @param updateReqVO 更新信息
     */
    void updateProductAttr(@Valid ProductAttrUpdateReqVO updateReqVO);

    /**
     * 删除产品属性
     *
     * @param id 编号
     */
    void deleteProductAttr(Long id);

    /**
     * 获得产品属性
     *
     * @param id 编号
     * @return 产品属性
     */
    ProductAttrDO getProductAttr(Long id);

    /**
     * 获得产品属性列表
     *
     * @return 产品属性列表
     */
    List<ProductAttrDO> getProductAttrList();

    /**
     * 获得产品属性分页
     *
     * @param pageReqVO 分页查询
     * @return 产品属性分页
     */
    PageResult<ProductAttrDO> getProductAttrPage(ProductAttrPageReqVO pageReqVO);

    /**
     * 获得产品属性列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 产品属性列表
     */
    List<ProductAttrDO> getProductAttrList(ProductAttrExportReqVO exportReqVO);

    List<ProductAttrDO> getAttrByIds(List<Long> attrIdList);
    List<Long> getAllAttrId();


    String delInvalidAttrIds(String attrIds);
}
