package cn.iocoder.yudao.module.product.service.product;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductBatchLogDO;
import cn.iocoder.yudao.module.product.vo.product.ProductBatchLogCreateReqVO;
import cn.iocoder.yudao.module.product.vo.product.ProductBatchLogQueryVO;
import cn.iocoder.yudao.module.product.vo.product.ProductBatchLogUpdateReqVO;

/**
 * 产品批量操作日志 Service 接口
 *
 * @author wlh
 */
public interface ProductBatchLogService extends IService<ProductBatchLogDO> {

    /**
     * 创建产品批量操作日志
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProductBatchLog(@Valid ProductBatchLogCreateReqVO createReqVO);

    /**
     * 更新产品批量操作日志
     * @param updateReqVO 更新信息
     */
    void updateProductBatchLog(@Valid ProductBatchLogUpdateReqVO updateReqVO);

    /**
     * 删除产品批量操作日志
     * @param id 编号
     */
    void deleteProductBatchLog(Long id);

    /**
     * 获得产品批量操作日志
     * @param id 编号
     * @return 产品批量操作日志
     */
    ProductBatchLogDO getProductBatchLog(Long id);

    /**
     * 获得产品批量操作日志列表
     * @param ids 编号
     * @return 产品批量操作日志列表
     */
    List<ProductBatchLogDO> getProductBatchLogList(Collection<Long> ids);

    /**
     * 获得产品批量操作日志分页
     * @param page 分页查询
     * @param query 查询
     * @return 产品批量操作日志分页
     */
    PageResult<ProductBatchLogDO> getProductBatchLogPage(ProductBatchLogQueryVO query, PageVO page);

    /**
     * 获得产品批量操作日志列表, 用于 Excel 导出
     * @param query 查询
     * @return 产品批量操作日志列表
     */
    List<ProductBatchLogDO> getProductBatchLogList(ProductBatchLogQueryVO query);
}
