package cn.iocoder.yudao.module.product.service.product;

import java.math.BigDecimal;
import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.BasePrice;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceStepDO;
import cn.iocoder.yudao.module.product.dto.AirClearancePriceResultDto;
import cn.iocoder.yudao.module.product.dto.CopyProductPriceDto;
import cn.iocoder.yudao.module.product.dto.ProductPriceExcelData;
import cn.iocoder.yudao.module.product.param.PriceParam;
import cn.iocoder.yudao.module.product.vo.productPrice.*;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceDO;
import org.apache.ibatis.annotations.Param;

/**
 * 产品价格 Service 接口
 *
 * @author wlh
 */
public interface ProductPriceService extends IService<ProductPriceDO> {

    /**
     * 创建产品价格
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProductPrice(@Valid ProductPriceCreateReqVO createReqVO, List<WarehouseLineDO> warehouseLineList);

    /**
     * 创建产品价格
     * @param createReqVO 创建信息
     * @return
     */
    Long singleCreate(ProductPriceCreateReqVO createReqVO);

    Long singleCreateAir(ProductPriceAirCreateReqVO createReqVO);

    /**
     * 更新产品价格
     *
     * @param updateReqVO 更新信息
     */
    void updateProductPrice(@Valid ProductPriceUpdateReqVO updateReqVO);

    /**
     * 删除产品价格
     *
     * @param id 编号
     */
    void deleteProductPrice(Long id);

    /**
     * 获得产品价格
     *
     * @param id 编号
     * @return 产品价格
     */
    ProductPriceDO getProductPrice(Long id);

    ProductPriceRespVO getProductPriceDetail(Long id);

    ProductPriceRespVO getProductPriceDetail(PriceParam priceParam);

    void matchLineAndProduct(List<ProductPriceRespVO> list);

    /**
     * 获得产品价格列表
     *
     * @param ids 编号
     * @return 产品价格列表
     */
    List<ProductPriceDO> getProductPriceList(Collection<Long> ids);

    /**
     * 批量下架
     * @param ids
     */
    void batchOff(Collection<Long> ids);

    /**
     * 批量拉黑
     * @param ids
     */
    void batchBlock(Collection<Long> ids);

    /**
     * 获得产品价格分页
     *
     * @param pageReqVO 分页查询
     * @return 产品价格分页
     */
    PageResult<ProductPriceDO> getProductPricePage(ProductPricePageReqVO pageReqVO);

    /**
     * 获得产品价格列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 产品价格列表
     */
    List<ProductPriceDO> getProductPriceList(ProductPriceExportReqVO exportReqVO);

    List<ProductPriceExcelData> getExcelList(int pageSize, Long maxId, ProductPricePageReqVO params);

    long getExcelListCount(ProductPricePageReqVO params);

    /**
     * 批量修改路线价格
     * @param updateReqVO
     */
    void batchUpdateProductPrice(ProductPriceBatchUpdateReqVo updateReqVO, ProductPriceCheckAO productPriceCheckAO);

    /**
     * 批量修改空运路线价格
     * @param updateReqVO
     */
    void batchUpdateProductPriceAir(ProductPriceBatchUpdateAirReqVo updateReqVO, ProductPriceCheckAO productPriceCheckAO);


    ProductPriceDO getProductPriceById(Long proId, Long lineId, Long channelId);

    ProductPriceCheckAO check(ProductPriceReqBaseVO productPriceReqBaseVO);

    void clearKey();

    void copy();

    void copyV1();

    void updateOrderInquiry(ProductPriceUpdateReqVO updateReqVO);

    /**
     * 路线价格复制
     * @param transportType  运输方式
     * @param productId     价格模版对应的商品ID
     * @param productIdList  需要复制的商品ID列表
     */
    void productPriceCopy(String transportType, Long productId, List<Long> productIdList);

    /**
     * 查询空运清关费
     * @param productId  商品ID
     * @param packaging  包装类型
     * @param channelId  渠道ID
     * @param weight   重量
     * @param volume  体积
     * @param quantity  数量
     * @return  AirClearancePriceResultDto,未匹配上clearancePrice为null
     */
    AirClearancePriceResultDto queryAirClearancePrice(Long productId, String packaging, Long channelId, BigDecimal weight, BigDecimal volume, BigDecimal quantity);


    /**
     * 查询空运清关费
     * @param productId  商品ID
     * @param packages  包装类型列表
     * @param channelId  渠道ID
     * @param weight   重量
     * @param volume  体积
     * @param quantity  数量
     * @return  AirClearancePriceResultDto,未匹配上clearancePrice为null
     */
    AirClearancePriceResultDto queryAirClearancePrice(Long productId, List<String> packages, Long channelId, BigDecimal weight, BigDecimal volume, BigDecimal quantity);

    void batchAddPrice(ProductPriceBatchAddPriceReqVo batchAddPriceReqVo, ProductPriceCheckAO productPriceCheckAO);

    /**
     * 根据产品和运输类型查询有效价格总数。（请注意内部多表关联）
     *
     * @param productId         产品id
     * @param transportTypeList 运输类型集合
     * @return
     */
    Long countByProductIdAndTransportType(Long productId, List<String> transportTypeList);

    /**
     * 渠道价格数量
     * @param channelId  渠道ID
     * @return
     */
    Long channelPriceCount(Long channelId);

    void updateProductPriceAir(ProductPriceAirUpdateReqVO updateReqVO);

    void initDictParams(ProductPricePageReqVO pageParam);

    void batchAddPriceAir(ProductPriceBatchAddPriceAirVo batchAddPriceReqVo, ProductPriceCheckAO productPriceCheckAO);

    void handleOrderException(Long priceId);

    /**
     * 从商品列表中某一商品进入复制价格
     * @param dto
     */
    void copyPriceByProductIds(CopyProductPriceDto dto);
}
