package cn.iocoder.yudao.module.product.service.product;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeCreateReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeExportReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypePageReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductTypeDO;

/**
 * 产品类型 Service 接口
 *
 * @author wlh
 */
public interface ProductTypeService {

    /**
     * 创建产品类型
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProductType(@Valid ProductTypeCreateReqVO createReqVO);

    /**
     * 更新产品类型
     *
     * @param updateReqVO 更新信息
     */
    void updateProductType(@Valid ProductTypeUpdateReqVO updateReqVO);

    /**
     * 删除产品类型
     *
     * @param id 编号
     */
    void deleteProductType(Long id);

    /**
     * 获得产品类型
     *
     * @param id 编号
     * @return 产品类型
     */
    ProductTypeDO getProductType(Long id);


    /**
     * 获得产品类型分页
     *
     * @param pageReqVO 分页查询
     * @return 产品类型分页
     */
    PageResult<ProductTypeDO> getProductTypePage(ProductTypePageReqVO pageReqVO);

    /**
     * 获得产品类型列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 产品类型列表
     */
    List<ProductTypeDO> getProductTypeList(ProductTypeExportReqVO exportReqVO);

}
