package cn.iocoder.yudao.module.product.service.product.impl;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseLineMapper;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseMapper;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceDO;
import cn.iocoder.yudao.module.product.dto.ProductPriceExcel;
import cn.iocoder.yudao.module.product.service.product.ProductPriceService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.metadata.CellExtra;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class ProductPriceExcelReadListener implements ReadListener<ProductPriceExcel> {
    @Autowired
    private ProductPriceService productPriceService;
    @Autowired
    private WarehouseLineMapper warehouseLineMapper;
    @Autowired
    private WarehouseMapper warehouseMapper;


    private static final int BATCH_COUNT = 10000;

    private static final Map<String, Long> warehouseMap = new HashMap<>();
    private static final Map<String, Long> lineIdMap = new HashMap<>();

    /**
     * 缓存的数据
     */
    private List<ProductPriceExcel> cachedDataList = new ArrayList<>(BATCH_COUNT);


    @Override
    public void onException(Exception e, AnalysisContext analysisContext) throws Exception {
//        e.printStackTrace();
    }

    @Override
    public void invokeHead(Map<Integer, CellData> headMap, AnalysisContext analysisContext) {
//        log.info("解析到一条头数据:{}", JSON.toJSONString(headMap));
    }

    @Override
    public void invoke(ProductPriceExcel productPriceExcel, AnalysisContext analysisContext) {
//        log.info("解析到一条数据:{}", JSON.toJSONString(productPriceExcel));
        cachedDataList.add(productPriceExcel);
//        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (cachedDataList.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            cachedDataList = new ArrayList<>(BATCH_COUNT);
        }
    }

    private void saveData() {
        List<ProductPriceDO> productList = new ArrayList<>();
        for (int i = 0; i < cachedDataList.size(); i++) {
            ProductPriceExcel item = cachedDataList.get(i);
            if(item.getProductId() == null) {
                continue;
            }
            Long productId = Long.valueOf(item.getProductId());

            ProductPriceDO productPriceDO = new ProductPriceDO();
            productPriceDO.setProductId(productId);

            String transportTypeName = item.getTransportType();
            if(transportTypeName == null) continue;
            //获取线路ID
            String transportType = ProductPriceExcel.transportTypeConvert(transportTypeName);
            String startWarehouseName = item.getStartWarehouseName();
            if(startWarehouseName == null) continue;
            String endWarehouseName = item.getEndWarehouseName();
            if(endWarehouseName == null) continue;

            Long startId;
            Long endId;
            if(warehouseMap.containsKey(startWarehouseName)) {
                startId = warehouseMap.get(startWarehouseName);
            } else {
                WarehouseDO warehouseDO = warehouseMapper.selectOne("title_zh", startWarehouseName);
                startId = warehouseDO.getId();
                warehouseMap.put(startWarehouseName, startId);
            }
            if(warehouseMap.containsKey(endWarehouseName)) {
                endId = warehouseMap.get(endWarehouseName);
            } else {
                WarehouseDO warehouseDO = warehouseMapper.selectOne("title_zh", endWarehouseName);
                endId = warehouseDO.getId();
                warehouseMap.put(endWarehouseName, endId);
            }
            Long lineID;
            String key = startId + "-" + endId + "-" + transportType;
            if(lineIdMap.containsKey(key)) {
                lineID = lineIdMap.get(key);
            } else {
                //找出路线
                WarehouseLineDO warehouseLineDO = warehouseLineMapper.findOne(startId, endId, transportType);
                lineID = warehouseLineDO.getId();
                lineIdMap.put(key, lineID);
            }

            productPriceDO.setWarehouseLineId(lineID);
            productPriceDO.setShippingChannelId(0L);

            String transportPrice = item.getTransportPrice();
            BigDecimal tPrice = BigDecimal.ZERO;
            if(transportPrice != null) {
                tPrice = new BigDecimal(transportPrice);
            }
            productPriceDO.setTransportPrice(tPrice);
            String transportPriceUintName = item.getTransportPriceUintName();
            productPriceDO.setTransportPriceUnit(ProductPriceExcel.priceUnitConvert(transportPriceUintName));
            String transportVolumeUnitName = item.getTransportVolumeUnitName();
            productPriceDO.setTransportVolumeUnit(ProductPriceExcel.volumeUnitConvert(transportVolumeUnitName, transportType));


            String clearancePrice = item.getClearancePrice();
            BigDecimal cPrice = BigDecimal.ZERO;
            if(clearancePrice != null) {
                cPrice = new BigDecimal(clearancePrice);
            }
            productPriceDO.setClearancePrice(cPrice);
            String clearancePriceUintName = item.getClearancePriceUintName();
            productPriceDO.setClearancePriceUnit(ProductPriceExcel.priceUnitConvert(clearancePriceUintName));
            String clearanceVolumeUnitName = item.getClearanceVolumeUnitName();
            productPriceDO.setClearanceVolumeUnit(ProductPriceExcel.volumeUnitConvert(clearanceVolumeUnitName, transportType));

            productPriceDO.setAllVolumeUnit(ProductPriceExcel.volumeUnitConvert(null, transportType));
            productPriceDO.setAuditStatus(ProductPriceExcel.auditConvert(item.getAuditType()));
            String needPayType = item.getNeedPayType();
            if(needPayType == null) continue;
            productPriceDO.setNeedPay(ProductPriceExcel.needPayConvert(needPayType));

            productList.add(productPriceDO);
        }
        log.info("批量添加");
        productPriceService.saveBatch(productList);
    }



    @Override
    public void extra(CellExtra cellExtra, AnalysisContext analysisContext) {
        log.info("extra");
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        saveData();
        log.info("所有数据解析完成！");
    }

    @Override
    public boolean hasNext(AnalysisContext analysisContext) {
        return true;
    }

    public static void main(String[] args) {
        String filePath = "D:\\product_price.xlsx";
        File file = new File(filePath);
        EasyExcel.read(file, ProductPriceExcel.class, new ProductPriceExcelReadListener())
        .sheet().doRead();

    }
}
