package cn.iocoder.yudao.module.product.service.product.impl;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.product.convert.productPriceStepClearance.ProductPriceStepClearanceConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceStepClearanceDO;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductPriceStepClearanceMapper;
import cn.iocoder.yudao.module.product.service.product.ProductPriceStepClearanceService;
import cn.iocoder.yudao.module.product.vo.product.ProductPriceStepClearanceBackVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 产品请关费阶梯价格 Service 实现类
 *
 * @author jiuping
 */
@Service
@Validated
public class ProductPriceStepClearanceServiceImpl extends AbstractService<ProductPriceStepClearanceMapper, ProductPriceStepClearanceDO> implements ProductPriceStepClearanceService {

    @Resource
    private ProductPriceStepClearanceMapper productPriceStepClearanceMapper;


    @Override
    public List<ProductPriceStepClearanceBackVO> getProductPriceStepClearanceList(Long productId) {
        List<ProductPriceStepClearanceDO> productPriceStepClearanceDOList = productPriceStepClearanceMapper.selectList(new LambdaQueryWrapperX<ProductPriceStepClearanceDO>()
                .eq(ProductPriceStepClearanceDO::getProductId, productId)
                .eq(ProductPriceStepClearanceDO::getDeleted, 0)
                .orderByAsc(ProductPriceStepClearanceDO::getStartNum)
        );
        return ProductPriceStepClearanceConvert.INSTANCE.convertList(productPriceStepClearanceDOList);
    }

    @Override
    public List<ProductPriceStepClearanceBackVO> getProductPriceStepClearanceList(List<Long> idList) {
        List<ProductPriceStepClearanceDO> productPriceStepClearanceDOList = productPriceStepClearanceMapper.selectList(new LambdaQueryWrapperX<ProductPriceStepClearanceDO>()
                .in(ProductPriceStepClearanceDO::getProductId, idList)
                .eq(ProductPriceStepClearanceDO::getDeleted, 0)
        );
        return ProductPriceStepClearanceConvert.INSTANCE.convertList(productPriceStepClearanceDOList);
    }

    @Override
    public ProductPriceStepClearanceDO getProductPriceStepClearance(Long productId, BigDecimal num) {
        return this.getOne(new LambdaQueryWrapperX<ProductPriceStepClearanceDO>()
                .eq(ProductPriceStepClearanceDO::getProductId, productId)
                .le(ProductPriceStepClearanceDO::getStartNum, num)
                .ge(ProductPriceStepClearanceDO::getEndNum, num)
                .eq(ProductPriceStepClearanceDO::getDeleted, 0)
                .orderByDesc(ProductPriceStepClearanceDO::getId)
                .last("limit 1")
        );
    }
}
