package cn.iocoder.yudao.module.product.service.product.impl;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeCreateReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeExportReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypePageReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.product.ProductTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductTypeDO;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductTypeMapper;
import cn.iocoder.yudao.module.product.service.product.ProductTypeService;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 产品类型 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class ProductTypeServiceImpl implements ProductTypeService {

    @Resource
    private ProductTypeMapper productTypeMapper;

    @Override
    public Long createProductType(ProductTypeCreateReqVO createReqVO) {
        // 插入
        ProductTypeDO productType = ProductTypeConvert.INSTANCE.convert(createReqVO);
        productTypeMapper.insert(productType);
        // 返回
        return productType.getId();
    }

    @Override
    public void updateProductType(ProductTypeUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateProductTypeExists(updateReqVO.getId());
        // 更新
        ProductTypeDO updateObj = ProductTypeConvert.INSTANCE.convert(updateReqVO);
        productTypeMapper.updateById(updateObj);
    }

    @Override
    public void deleteProductType(Long id) {
        // 校验存在
        this.validateProductTypeExists(id);
        // 删除
        productTypeMapper.deleteById(id);
    }

    private void validateProductTypeExists(Long id) {
        if (productTypeMapper.selectById(id) == null) {
            throw exception(PRODUCT_TYPE_NOT_EXISTS);
        }
    }

    @Override
    public ProductTypeDO getProductType(Long id) {
        return productTypeMapper.selectById(id);
    }

    @Override
    public PageResult<ProductTypeDO> getProductTypePage(ProductTypePageReqVO pageReqVO) {
        return productTypeMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ProductTypeDO> getProductTypeList(ProductTypeExportReqVO exportReqVO) {
        return productTypeMapper.selectList(exportReqVO);
    }

}
