package cn.iocoder.yudao.module.product.service.productBrandTrademarkRel;

import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.module.product.convert.productBrandTrademarkRel.ProductBrandTrademarkRelConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.productBrandTrademarkRel.ProductBrandTrademarkRelDO;
import cn.iocoder.yudao.module.product.dal.mysql.productBrandTrademarkRel.ProductBrandTrademarkRelMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.product.vo.productBrandTrademarkRel.ProductBrandTrademarkRelCreateReqVO;
import cn.iocoder.yudao.module.product.vo.productBrandTrademarkRel.ProductBrandTrademarkRelExportReqVO;
import cn.iocoder.yudao.module.product.vo.productBrandTrademarkRel.ProductBrandTrademarkRelPageReqVO;
import cn.iocoder.yudao.module.product.vo.productBrandTrademarkRel.ProductBrandTrademarkRelUpdateReqVO;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.product.vo.productBrandTrademarkRel.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;

/**
 * 商品品牌与商标关联 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class ProductBrandTrademarkRelServiceImpl extends AbstractService<ProductBrandTrademarkRelMapper, ProductBrandTrademarkRelDO> implements ProductBrandTrademarkRelService {

    @Resource
    private ProductBrandTrademarkRelMapper productBrandTrademarkRelMapper;

    @Override
    public Long createProductBrandTrademarkRel(ProductBrandTrademarkRelCreateReqVO createReqVO) {
        // 插入
        ProductBrandTrademarkRelDO productBrandTrademarkRel = ProductBrandTrademarkRelConvert.INSTANCE.convert(createReqVO);
        productBrandTrademarkRelMapper.insert(productBrandTrademarkRel);
        // 返回
        return productBrandTrademarkRel.getId();
    }

    @Override
    public void updateProductBrandTrademarkRel(ProductBrandTrademarkRelUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateProductBrandTrademarkRelExists(updateReqVO.getId());
        // 更新
        ProductBrandTrademarkRelDO updateObj = ProductBrandTrademarkRelConvert.INSTANCE.convert(updateReqVO);
        productBrandTrademarkRelMapper.updateById(updateObj);
    }

    @Override
    public void deleteProductBrandTrademarkRel(Long id) {
        // 校验存在
        this.validateProductBrandTrademarkRelExists(id);
        // 删除
        productBrandTrademarkRelMapper.deleteById(id);
    }

    private void validateProductBrandTrademarkRelExists(Long id) {
        if (productBrandTrademarkRelMapper.selectById(id) == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.PRODUCT_BRAND_TRADEMARK_REL_NOT_EXISTS);
        }
    }

    @Override
    public ProductBrandTrademarkRelDO getProductBrandTrademarkRel(Long id) {
        return productBrandTrademarkRelMapper.selectById(id);
    }

    @Override
    public List<ProductBrandTrademarkRelDO> getProductBrandTrademarkRelList(Collection<Long> ids) {
        return productBrandTrademarkRelMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ProductBrandTrademarkRelDO> getProductBrandTrademarkRelPage(ProductBrandTrademarkRelPageReqVO pageReqVO) {
        return productBrandTrademarkRelMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ProductBrandTrademarkRelDO> getProductBrandTrademarkRelList(ProductBrandTrademarkRelExportReqVO exportReqVO) {
        return productBrandTrademarkRelMapper.selectList(exportReqVO);
    }

}
