package cn.iocoder.yudao.module.product.service.productPriceSnapshot;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.product.convert.productPriceSnapshot.ProductPriceSnapshotConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.productPriceSnapshot.ProductPriceSnapshotDO;
import cn.iocoder.yudao.module.product.dal.mysql.productPriceSnapshot.ProductPriceSnapshotMapper;
import cn.iocoder.yudao.module.product.vo.productPriceSnapshot.ProductPriceSnapshotCreateReqVO;
import cn.iocoder.yudao.module.product.vo.productPriceSnapshot.ProductPriceSnapshotQueryVO;
import cn.iocoder.yudao.module.product.vo.productPriceSnapshot.ProductPriceSnapshotUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 路线价格修改历史 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class ProductPriceSnapshotServiceImpl extends AbstractService<ProductPriceSnapshotMapper, ProductPriceSnapshotDO> implements ProductPriceSnapshotService {

    @Resource
    private ProductPriceSnapshotMapper productPriceSnapshotMapper;

    @Override
    public List<ProductPriceSnapshotDO> getByPriceId(Long priceId) {
        return productPriceSnapshotMapper.selectList(new LambdaQueryWrapper<ProductPriceSnapshotDO>()
            .eq(ProductPriceSnapshotDO::getPriceId, priceId)
                .orderByDesc(ProductPriceSnapshotDO::getId)
        );
    }

    @Override
    public Long createProductPriceSnapshot(ProductPriceSnapshotCreateReqVO createReqVO) {
        // 插入
        ProductPriceSnapshotDO productPriceSnapshot = ProductPriceSnapshotConvert.INSTANCE.convert(createReqVO);
        productPriceSnapshotMapper.insert(productPriceSnapshot);
        // 返回
        return productPriceSnapshot.getId();
    }

    @Override
    public void updateProductPriceSnapshot(ProductPriceSnapshotUpdateReqVO updateReqVO) {
        // 更新
        ProductPriceSnapshotDO updateObj = ProductPriceSnapshotConvert.INSTANCE.convert(updateReqVO);
        productPriceSnapshotMapper.updateById(updateObj);
    }

    @Override
    public void deleteProductPriceSnapshot(Long id) {
        // 删除
        productPriceSnapshotMapper.deleteById(id);
    }


    @Override
    public ProductPriceSnapshotDO getProductPriceSnapshot(Long id) {
        return productPriceSnapshotMapper.selectById(id);
    }

    @Override
    public List<ProductPriceSnapshotDO> getProductPriceSnapshotList(Collection<Long> ids) {
        return productPriceSnapshotMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ProductPriceSnapshotDO> getProductPriceSnapshotPage(ProductPriceSnapshotQueryVO query, PageVO page) {
        return productPriceSnapshotMapper.selectPage(page, query);
    }

    @Override
    public List<ProductPriceSnapshotDO> getProductPriceSnapshotList(ProductPriceSnapshotQueryVO query) {
        return productPriceSnapshotMapper.selectList(query);
    }
}
