package cn.iocoder.yudao.module.product.service.productbrandempower;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.product.vo.productbrandempower.*;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrandempower.ProductBrandEmpowerDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.product.vo.productbrandempower.*;

/**
 * 商品品牌授权 Service 接口
 *
 * @author 江楠
 */
public interface ProductBrandEmpowerService extends IService<ProductBrandEmpowerDO> {

    /**
     * 创建商品品牌授权
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProductBrandEmpower(@Valid ProductBrandEmpowerCreateReqVO createReqVO);

    /**
     * 更新商品品牌授权
     *
     * @param updateReqVO 更新信息
     */
    void updateProductBrandEmpower(@Valid ProductBrandEmpowerUpdateReqVO updateReqVO);

    /**
     * 删除商品品牌授权
     *
     * @param id 编号
     */
    void deleteProductBrandEmpower(Long id);

    /**
     * 获得商品品牌授权
     *
     * @param id 编号
     * @return 商品品牌授权
     */
    ProductBrandEmpowerDO getProductBrandEmpower(Long id);

    /**
     * 获取客户针对商品的备案信息
     * 品牌是否备案：
     * 1.无品牌
     * 2.有品牌：有品牌、有备案
     * 3.中性：仿牌
     *
     * 价格计算规则：
     * 1.默认线路运费为无牌价格
     * 2.如果入仓时选择商品，查询商品是否关联品牌信息：
     *    a.该品牌未在系统内添加，则按路线产品无牌价计算。
     *    b.该品牌在系统内添加，备案类型为中性，该品牌未对客户授权，则按路线产品中性价计算。
     *    c.该品牌在系统内容已添加，备案类型为有牌，该品牌未对客户授权，则按路线产品有牌价计算。
     *    d.该品牌在系统内添加，备案类型为中性或有牌，该品牌已对客户授权，则按客户授权时设定的收费模式（有牌价或中性价、无牌价）计费。
     *     e.如果品牌设置了商标，并限定了品牌，还需要检查入仓商品是否在品牌商标范围内，若不在范围内，按无牌价计算。
     * @return
     */
    GetFeeTypeProductBrandEmpowerRespVO getFeeTypeProductBrandEmpower(GetFeeTypeProductBrandEmpowerReqVO reqVO);

    /**
     * 根据订单id查询客户id
     * @param reqVO
     * @return
     */
    GetFeeTypeProductBrandEmpowerRespVO getFeeTypeByOrder(GetFeeTypeProductBrandByOrderEmpowerReqVO reqVO);

    /**
     * 获得商品品牌授权列表
     *
     * @param ids 编号
     * @return 商品品牌授权列表
     */
    List<ProductBrandEmpowerDO> getProductBrandEmpowerList(Collection<Long> ids);
    List<ProductBrandEmpowerDO> getProductBrandEmpowerList(ProductBrandEmpowerBaseVO empowerBaseVO);

    /**
     * 获得商品品牌授权分页
     *
     * @param pageReqVO 分页查询
     * @return 商品品牌授权分页
     */
    PageResult<ProductBrandEmpowerDO> getProductBrandEmpowerPage(ProductBrandEmpowerPageReqVO pageReqVO);

    /**
     * 获取品牌的已授权客户列表
     * @param pageReqVO
     * @return
     */
    PageResult<ProductBrandEmpowerRespVO> getProductBrandEmpowerCustomerPage(ProductBrandEmpowerPageReqVO pageReqVO);

    /**
     * 获取品牌的未授权客户列表
     * @param pageReqVO
     * @return
     */
    PageResult<ProductBrandEmpowerRespVO> getProductBrandEmpowerUnAuthCustomerPage(ProductBrandEmpowerPageReqVO pageReqVO);

    /**
     * 获得商品品牌授权列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 商品品牌授权列表
     */
    List<ProductBrandEmpowerDO> getProductBrandEmpowerList(ProductBrandEmpowerExportReqVO exportReqVO);

}
