package cn.iocoder.yudao.module.product.service.productbrank;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.product.vo.productbrank.*;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.product.vo.productbrank.ProductBrankCreateReqVO;
import cn.iocoder.yudao.module.product.vo.productbrank.ProductBrankExportReqVO;
import cn.iocoder.yudao.module.product.vo.productbrank.ProductBrankPageReqVO;
import cn.iocoder.yudao.module.product.vo.productbrank.ProductBrankUpdateReqVO;

/**
 * 品牌 Service 接口
 *
 * @author 江楠
 */
public interface ProductBrankService extends IService<ProductBrankDO> {

    /**
     * 创建品牌
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProductBrank(@Valid ProductBrankCreateReqVO createReqVO);


    Long createIfNotPresent(String brandName);

    /**
     * 获取客户拥有授权的品牌信息
     * @param createReqVO
     * @return
     */
    PageResult<ProductBrandMyEmpowerRespVO> getBrankByCustomer(@Valid ProductBrankByCustomerPageReqVO createReqVO);

    /**
     * 更新品牌
     *
     * @param updateReqVO 更新信息
     */
    void updateProductBrank(@Valid ProductBrankUpdateReqVO updateReqVO);
    void changeProductBrankFeeMode(@Valid ProductBrankChangeFeeModeReqVO updateReqVO);

    /**
     * 删除品牌
     *
     * @param id 编号
     */
    void deleteProductBrank(Long id);

    /**
     * 获得品牌
     *
     * @param id 编号
     * @return 品牌
     */
    ProductBrankDO getProductBrank(Long id);

    /**
     * 获得品牌列表
     *
     * @param ids 编号
     * @return 品牌列表
     */
    List<ProductBrankDO> getProductBrankList(Collection<Long> ids);

    /**
     * 获得品牌分页
     *
     * @param pageReqVO 分页查询
     * @return 品牌分页
     */
    PageResult<ProductBrankDO> getProductBrankPage(ProductBrankPageReqVO pageReqVO);

    /**
     * 获得品牌列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 品牌列表
     */
    List<ProductBrankDO> getProductBrankList(ProductBrankExportReqVO exportReqVO);

}
