package cn.iocoder.yudao.module.product.controller.admin.coupon;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponProdDO;
import cn.iocoder.yudao.module.product.dto.CouponUserAvailableDto;
import cn.iocoder.yudao.module.product.param.AvailableCouponParam;
import cn.iocoder.yudao.module.product.param.CouponDelayTimeParam;
import cn.iocoder.yudao.module.product.param.CouponPutonStatusParam;
import cn.iocoder.yudao.module.product.service.coupon.CouponProdService;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import cn.iocoder.yudao.module.product.vo.coupon.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;

import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;

import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponDO;
import cn.iocoder.yudao.module.product.convert.coupon.CouponConvert;

@Api(tags = "管理后台 - 优惠券信息")
@RestController
@RequestMapping("/product/coupon")
@Validated
public class CouponController {

    @Resource
    private CouponService couponService;
    @Resource
    private CouponProdService couponProdService;

    @PostMapping("/createOrUpdate")
    @ApiOperation("创建优惠券信息")
    @Idempotent(timeout = 3)
    //@PreAuthorize("@ss.hasPermission('product:coupon:create')")
    public CommonResult<Long> createCoupon(@Valid @RequestBody CouponCreateReqVO createReqVO) {
        if (createReqVO.getType() != 1) {
            if (StringUtils.isBlank(createReqVO.getOrderAttr())) {
                return error(ORDER_TYPE_NOT_NULL);
            }
            if (StringUtils.isBlank(createReqVO.getBrand())) {
                return error(BRAND_TYPE_NOT_NULL);
            }

            if (StringUtils.isBlank(createReqVO.getIsCargoControl())) {
                return error(IS_CARGO_CONTROL_NOT_NULL);
            }
            if (StringUtils.isBlank(createReqVO.getDocumentDeclaration())) {
                return error(DOCUMENT_DECLARATION_NOT_NULL);
            }
        }
        return success(couponService.createCoupon(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新优惠券信息")
    @Idempotent(timeout = 3)
    //@PreAuthorize("@ss.hasPermission('product:coupon:update')")
    public CommonResult<Boolean> updateCoupon(@Valid @RequestBody CouponUpdateReqVO updateReqVO) {
        if (updateReqVO.getType() != 1) {
            if (StringUtils.isBlank(updateReqVO.getOrderAttr())) {
                return error(ORDER_TYPE_NOT_NULL);
            }
            if (StringUtils.isBlank(updateReqVO.getBrand())) {
                return error(BRAND_TYPE_NOT_NULL);
            }

            if (StringUtils.isBlank(updateReqVO.getIsCargoControl())) {
                return error(IS_CARGO_CONTROL_NOT_NULL);
            }
            if (StringUtils.isBlank(updateReqVO.getDocumentDeclaration())) {
                return error(DOCUMENT_DECLARATION_NOT_NULL);
            }
        }
        couponService.updateCoupon(updateReqVO);
        return success(true);
    }

    @PutMapping("/update/endTime")
    @ApiOperation("更新优惠券有效期")
    public CommonResult<Boolean> updateCouponEndTime(@Valid @RequestBody CouponDelayTimeParam param) {
        couponService.updateCouponEndTime(param.getCouponId(), param.getEndTime(), param.getIsValidity());
        return success(true);
    }

    @PutMapping("/update/putonStatus")
    @ApiOperation("更新优惠投放状态")
    public CommonResult<Boolean> updateCouponPutonStatus(@Valid @RequestBody CouponPutonStatusParam param) {
        couponService.updateCouponPutonStatus(param.getCouponId(), param.getPutonStatus());
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除优惠券信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('product:coupon:delete')")
    public CommonResult<Boolean> deleteCoupon(@RequestParam("id") Long id) {
        couponService.deleteCoupon(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得优惠券信息")
    @ApiImplicitParam(name = "couponId", value = "优惠劵ID", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('product:coupon:query')")
    public CommonResult<CouponRespVO> getCoupon(@RequestParam("couponId") Long couponId) {
        CouponRespVO couponRespVO = couponService.getInfo(couponId);
        return success(couponRespVO);
    }


    @PostMapping("/customer/available/couponList")
    @ApiOperation("获得用户可用优惠信息")
    @Deprecated
    //@PreAuthorize("@ss.hasPermission('product:coupon:query')")
    public CommonResult<CouponUserAvailableDto> getCouponUserAvailableDto(@RequestBody @Valid AvailableCouponParam availableCouponParam) {
//        if (null == availableCouponParam.getConsignorCustomerContactsId()) {
//            return error(CUSTOMER_CONTACTS_NOT_NULL);
//        }
        CouponUserAvailableDto dto = couponService.getCouponUserAvailableDto(availableCouponParam.getOrderType(),
                availableCouponParam.getCustomsType(), availableCouponParam.getIsCargoControl() ? 1 : 0, null,
                availableCouponParam.getConsigneeCustomerId(), availableCouponParam.getConsigneeCustomerId(),
                availableCouponParam.getProdConditionParamList(), availableCouponParam.getTransportId(), availableCouponParam.getLineId(),
                availableCouponParam.getChannelId());
        return success(dto);
    }

    @GetMapping("/list")
    @ApiOperation("获得优惠券信息列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('product:coupon:query')")
    public CommonResult<List<CouponRespVO>> getCouponList(@RequestParam("ids") Collection<Long> ids) {
        List<CouponDO> list = couponService.getCouponList(ids);
        return success(CouponConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/select")
    @ApiOperation("获得优惠券信息列表下拉框")
    //@PreAuthorize("@ss.hasPermission('product:coupon:query')")
    public CommonResult<List<CouponRespVO>> getCouponSelect(CouponPageReqVO pageVO) {
        List<CouponDO> couponDOList = couponService.getCouponSelect(pageVO);
        return success(CouponConvert.INSTANCE.convertList(couponDOList));
    }


    @GetMapping("/newProd/list")
    @ApiOperation("获得新增商品可用优惠信息列表")
    public CommonResult<List<CouponProdRespVO>> getNewProdCouponList(@RequestParam(value = "prodId", required = false) Long prodId) {
        List<CouponDO> couponDOList = couponService.getNewProdCouponList();
        List<CouponProdDO> prodDOList = null;
        if (Objects.nonNull(prodId) && prodId > 0L){
            prodDOList = couponProdService.list(new LambdaQueryWrapper<CouponProdDO>().eq(CouponProdDO::getProdId, prodId));
        }
        List<CouponProdRespVO> list = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(couponDOList)){
            for (CouponDO c : couponDOList) {
                CouponProdRespVO respVO = new CouponProdRespVO();
                BeanUtils.copyProperties(c, respVO);

                respVO.setMandatory(respVO.getSuitableProdType() == 0);// 全部商品参与的优惠为必选优惠，因为不能改变优惠信息状态
                respVO.setIsDefault(respVO.getSuitableProdType() != 1);// 不是指定商品参与的优惠为默认参与
                if (Objects.nonNull(prodDOList) && CollectionUtil.isNotEmpty(prodDOList) && respVO.getSuitableProdType() != 0){
                    // 编辑时有id，做比较
                    long count = prodDOList.stream().filter(p-> Objects.equals(p.getCouponId(), c.getCouponId())).count();
                    if (count > 0){
                        respVO.setMandatory(true);// 已经加入优惠条件的商品不可改变状态
                        respVO.setIsDefault(respVO.getSuitableProdType() == 1); // 指定参与就默认，不指定参非默认
                    }
                }
                list.add(respVO);
            }
        }
        return success(list);
    }

    @GetMapping("/page")
    @ApiOperation("获得优惠券信息分页")
    //@PreAuthorize("@ss.hasPermission('product:coupon:query')")
    public CommonResult<PageResult<CouponRespVO>> getCouponPage(@Valid CouponPageReqVO pageVO) {
        PageResult<CouponDO> pageResult = couponService.getCouponPage(pageVO);
        return success(CouponConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出优惠券信息 Excel")
    //@PreAuthorize("@ss.hasPermission('product:coupon:export')")
    @OperateLog(type = EXPORT)
    public void exportCouponExcel(@Valid CouponExportReqVO exportReqVO,
                                  HttpServletResponse response) throws IOException {
        List<CouponDO> list = couponService.getCouponList(exportReqVO);
        // 导出 Excel
        List<CouponExcelVO> datas = CouponConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "优惠券信息.xls", "数据", CouponExcelVO.class, datas);
    }

}
