package cn.iocoder.yudao.module.product.controller.admin.coupon;

import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordExcelVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordPageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordRespVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponUseRecordDO;
import cn.iocoder.yudao.module.product.convert.coupon.CouponUseRecordConvert;
import cn.iocoder.yudao.module.product.service.coupon.CouponUseRecordService;

@Api(tags = "管理后台 - 优惠券使用记录")
@RestController
@RequestMapping("/product/coupon-use-record")
@Validated
public class CouponUseRecordController {

    @Resource
    private CouponUseRecordService couponUseRecordService;


    @GetMapping("/get")
    @ApiOperation("获得优惠券使用记录")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:coupon-use-record:query')")
    public CommonResult<CouponUseRecordRespVO> getCouponUseRecord(@RequestParam("id") Long id) {
        CouponUseRecordDO couponUseRecord = couponUseRecordService.getCouponUseRecord(id);
        return success(CouponUseRecordConvert.INSTANCE.convert(couponUseRecord));
    }
//
//    @GetMapping("/list")
//    @ApiOperation("获得优惠券使用记录列表")
//    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    //@PreAuthorize("@ss.hasPermission('ecw:coupon-use-record:query')")
//    public CommonResult<List<CouponUseRecordRespVO>> getCouponUseRecordList(@RequestParam("ids") Collection<Long> ids) {
//        List<CouponUseRecordDO> list = couponUseRecordService.getCouponUseRecordList(ids);
//        return success(CouponUseRecordConvert.INSTANCE.convertList(list));
//    }

    @GetMapping("/page")
    @ApiOperation("获得优惠券使用记录分页")
    //@PreAuthorize("@ss.hasPermission('ecw:coupon-use-record:query')")
    public CommonResult<PageResult<CouponUseRecordRespVO>> getCouponUseRecordPage(@Valid CouponUseRecordPageReqVO pageVO) {
        PageResult<CouponUseRecordDO> pageResult = couponUseRecordService.getCouponUseRecordPage(pageVO);
        return success(CouponUseRecordConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出优惠券使用记录 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:coupon-use-record:export')")
    @OperateLog(type = EXPORT)
    public void exportCouponUseRecordExcel(@Valid CouponUseRecordExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<CouponUseRecordDO> list = couponUseRecordService.getCouponUseRecordList(exportReqVO);
        // 导出 Excel
        List<CouponUseRecordExcelVO> datas = CouponUseRecordConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "优惠券使用记录.xls", "数据", CouponUseRecordExcelVO.class, datas);
    }

}
