package cn.iocoder.yudao.module.product.controller.admin.product;

import cn.iocoder.yudao.module.product.vo.productAttr.*;
import cn.iocoder.yudao.module.product.convert.product.ProductAttrConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;


@Api(tags = "管理后台 - 商品属性")
@RestController
@RequestMapping("/ecw/product-attr")
@Validated
public class ProductAttrController {

    @Resource
    private ProductAttrService productAttrService;

    @PostMapping("/create")
    @ApiOperation("创建商品属性")
    @PreAuthorize("@ss.hasPermission('ecw:product-attr:create')")
    public CommonResult<Long> createProductAttr(@Valid @RequestBody ProductAttrCreateReqVO createReqVO) {
        return success(productAttrService.createProductAttr(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新产品属性")
    @PreAuthorize("@ss.hasPermission('ecw:product-attr:update')")
    public CommonResult<Boolean> updateProductAttr(@Valid @RequestBody ProductAttrUpdateReqVO updateReqVO) {
        productAttrService.updateProductAttr(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除商品属性")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:product-attr:delete')")
    public CommonResult<Boolean> deleteProductAttr(@RequestParam("id") Long id) {
        productAttrService.deleteProductAttr(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得商品属性")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:product-attr:query')")
    public CommonResult<ProductAttrRespVO> getProductAttr(@RequestParam("id") Long id) {
        ProductAttrDO productAttr = productAttrService.getProductAttr(id);
        return success(ProductAttrConvert.INSTANCE.convert(productAttr));
    }

    @GetMapping("/list")
    @ApiOperation("获得商品属性列表")
    public CommonResult<List<ProductAttrRespVO>> getProductAttrList() {
        List<ProductAttrDO> list = productAttrService.getProductAttrList();
        return success(ProductAttrConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得商品属性分页")
    @PreAuthorize("@ss.hasPermission('ecw:product-attr:query')")
    public CommonResult<PageResult<ProductAttrRespVO>> getProductAttrPage(@Valid ProductAttrPageReqVO pageVO) {
        PageResult<ProductAttrDO> pageResult = productAttrService.getProductAttrPage(pageVO);
        return success(ProductAttrConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出商品属性 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:product-attr:export')")
    @OperateLog(type = EXPORT)
    public void exportProductAttrExcel(@Valid ProductAttrExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<ProductAttrDO> list = productAttrService.getProductAttrList(exportReqVO);
        // 导出 Excel
        List<ProductAttrExcelVO> datas = ProductAttrConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "商品属性.xls", "数据", ProductAttrExcelVO.class, datas);
    }

}
