package cn.iocoder.yudao.module.product.controller.admin.product;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.product.convert.product.ProductPriceConvert;
import cn.iocoder.yudao.module.product.convert.productPriceSnapshot.ProductPriceSnapshotConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceDO;
import cn.iocoder.yudao.module.product.dal.dataobject.productPriceSnapshot.ProductPriceSnapshotDO;
import cn.iocoder.yudao.module.product.dto.CopyProductPriceDto;
import cn.iocoder.yudao.module.product.dto.ProductPriceExcel;
import cn.iocoder.yudao.module.product.param.PriceParam;
import cn.iocoder.yudao.module.product.service.product.ProductPriceService;
import cn.iocoder.yudao.module.product.service.product.impl.ProductPriceExcelReadListener;
import cn.iocoder.yudao.module.product.service.product.impl.RouterService;
import cn.iocoder.yudao.module.product.service.productPriceSnapshot.ProductPriceSnapshotService;
import cn.iocoder.yudao.module.product.vo.productPrice.*;
import cn.iocoder.yudao.module.product.vo.productPriceSnapshot.ProductPriceSnapshotBackVO;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.File;
import java.io.IOException;
import java.util.*;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;


@Api(tags = "管理后台 - 路线产品")
@RestController
@RequestMapping("/ecw/product-price")
@Validated
public class ProductPriceController {

    @Resource
    private ProductPriceService productPriceService;
    @Resource
    private RouterService routerService;
    @Resource
    private ProductPriceSnapshotService productPriceSnapshotService;

    @Resource
    private FileMakeApi fileMakeApi;

    @PostMapping("/create")
    @ApiOperation("创建路线产品")
    @PreAuthorize("@ss.hasPermission('ecw:product-price:create')")
    @Idempotent(timeout = 5)
    public CommonResult<Long> createProductPrice(
            @Valid @RequestBody ProductPriceCreateReqVO createReqVO) {
        return success(productPriceService.singleCreate(createReqVO));
    }

    @PostMapping("/createAir")
    @ApiOperation("创建路线产品空运")
    @Idempotent(timeout = 5)
    public CommonResult<Long> createProductPriceAir(
            @Valid @RequestBody ProductPriceAirCreateReqVO createReqVO) {
        return success(productPriceService.singleCreateAir(createReqVO));
    }

    @GetMapping("/clearKey")
    public CommonResult<Boolean> clearKey() {
        productPriceService.clearKey();
        return success(true);
    }

    @PutMapping("/update")
    @ApiOperation("更新路线产品")
    @PreAuthorize("@ss.hasPermission('ecw:product-price:update')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> updateProductPrice(
            @Valid @RequestBody ProductPriceUpdateReqVO updateReqVO) {
        productPriceService.updateProductPrice(updateReqVO);
        return success(true);
    }

    /*
     修改空运价格，在处理价格异常时，处理和此价格关联的订单信息
     */
    @PutMapping("/updateAir")
    @ApiOperation("更新空运路线产品")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> updateProductPriceAir(
            @Valid @RequestBody ProductPriceAirUpdateReqVO productPriceAirUpdateReqVO) {
        productPriceService.updateProductPriceAir(productPriceAirUpdateReqVO);
        return success(true);
    }

    @PutMapping("/updateOrderInquiry")
    @ApiOperation("更新单询")
    @PreAuthorize("@ss.hasPermission('ecw:product-price:update')")
    public CommonResult<Boolean> updateOrderInquiry(@Valid @RequestBody ProductPriceUpdateReqVO updateReqVO) {
        productPriceService.updateOrderInquiry(updateReqVO);
        return success(true);
    }

    @PutMapping("/batchUpdate")
    @ApiOperation("批量更新路线产品")
    @PreAuthorize("@ss.hasPermission('ecw:product-price:update')")
    public CommonResult<Boolean> batchUpdateProductPrice(@RequestBody ProductPriceBatchUpdateReqVo updateReqVO) {
        //批量修改价格
        //检查Redis中是否有正在修改的产品价格
        ProductPriceCheckAO productPriceCheckAO =
                productPriceService.check(updateReqVO);

        routerService.batchUpdateProductPrice(updateReqVO, productPriceCheckAO);
        return success(true);
    }

    @PutMapping("/batchUpdateAir")
    @ApiOperation("批量更新路线产品空运")
    public CommonResult<Boolean> batchUpdateProductPriceAir(@RequestBody ProductPriceBatchUpdateAirReqVo updateReqVO) {
        ProductPriceCheckAO productPriceCheckAO = productPriceService.check(updateReqVO);
        routerService.batchUpdateProductPriceAir(updateReqVO, productPriceCheckAO);
        return success(true);
    }

    @PutMapping("/batchAddPrice")
    @ApiOperation("批量加价")
    @PreAuthorize("@ss.hasPermission('ecw:product-price:update')")
    public CommonResult<Boolean> batchAddPrice(@RequestBody ProductPriceBatchAddPriceReqVo updateReqVO) {

        //批量修改价格
        updateReqVO.setCheckStepPrice(false);
        ProductPriceCheckAO productPriceCheckAO =
                productPriceService.check(updateReqVO);
        routerService.batchAddPrice(updateReqVO, productPriceCheckAO);
        return success(true);
    }

    @PutMapping("/batchAddPriceAir")
    @ApiOperation("批量加价空运")
    public CommonResult<Boolean> batchAddPriceAir(@RequestBody ProductPriceBatchAddPriceAirVo updateReqVO) {
        updateReqVO.setCheckStepPrice(false);
        ProductPriceCheckAO productPriceCheckAO = productPriceService.check(updateReqVO);
        routerService.batchAddPriceAir(updateReqVO, productPriceCheckAO);
        return success(true);
    }

    @GetMapping("/priceSnapshotList")
    @ApiOperation("线路价格日志")
    @ApiImplicitParam(name = "priceId", value = "路线价格ID", required = true, example = "1", dataTypeClass = Long.class)
    public CommonResult<List<ProductPriceSnapshotBackVO>> priceSnapshotList(@RequestParam("priceId") Long priceId) {
        List<ProductPriceSnapshotDO> priceSnapshotList = productPriceSnapshotService.getByPriceId(priceId);
        return success(ProductPriceSnapshotConvert.INSTANCE.convertList(priceSnapshotList));
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除路线产品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:product-price:delete')")
    public CommonResult<Boolean> deleteProductPrice(@RequestParam("id") Long id) {
        productPriceService.deleteProductPrice(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得路线产品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:product-price:query')")
    public CommonResult<ProductPriceRespVO> getProductPrice(@RequestParam("id") Long id) {
        ProductPriceRespVO productPriceRespVO = productPriceService.getProductPriceDetail(id);
        return success(productPriceRespVO);
    }

    @GetMapping("/getPrice")
    @ApiOperation("查询路线产品详情")
    public CommonResult<ProductPriceRespVO> getProductPrice(PriceParam priceParam) {
        ProductPriceRespVO productPriceRespVO = productPriceService.getProductPriceDetail(priceParam);
        return success(productPriceRespVO);
    }

    @GetMapping("/list")
    @ApiOperation("获得路线产品列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:product-price:query')")
    public CommonResult<List<ProductPriceRespVO>> getProductPriceList(@RequestParam("ids") Collection<Long> ids) {
        List<ProductPriceDO> doList = productPriceService.getProductPriceList(ids);
        List<ProductPriceRespVO> list = ProductPriceConvert.INSTANCE.convertList(doList);
        productPriceService.matchLineAndProduct(list);
        return success(list);
    }

    @GetMapping("/batchOff")
    @ApiOperation("批量下架")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('ecw:product-price:batchOff')")
    public CommonResult<Boolean> batchOff(@RequestParam("ids") Collection<Long> ids) {
        productPriceService.batchOff(ids);
        return success(true);
    }

    @GetMapping("/batchBlock")
    @ApiOperation("批量拉黑")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('ecw:product-price:batchBlock')")
    public CommonResult<Boolean> batchBlock(
            @RequestParam("ids") Collection<Long> ids) {
        productPriceService.batchBlock(ids);
        return success(true);
    }

    @GetMapping("/page")
    @ApiOperation("获得路线产品分页")
    @PreAuthorize("@ss.hasPermission('ecw:product-price:query')")
    public CommonResult<PageResult<ProductPriceRespVO>> getProductPricePage(
            @Valid ProductPricePageReqVO pageVO) {

        PageResult<ProductPriceDO> pageResult =
                productPriceService.getProductPricePage(pageVO);

        PageResult<ProductPriceRespVO> respPageResult =
                ProductPriceConvert.INSTANCE.convertPage(pageResult);
        productPriceService.matchLineAndProduct(respPageResult.getList());
        return success(respPageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出路线产品 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:product-price:export')")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportProductPriceExcel(@Valid ProductPricePageReqVO exportReqVO,
                                        HttpServletResponse response) throws IOException {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(DownloadTypeEnum.PRODUCT_PRICE_EXCEL_EXPORT.getType());
        reqDTO.setName("商品价格导出Excel");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(exportReqVO));
        fileMakeApi.sendFileMake(reqDTO);
        return CommonResult.success(true);
    }

    @Resource
    ProductPriceExcelReadListener productPriceExcelReadListener;

    @GetMapping("/excelImport")
    public void excelImport() {
        String filePath = "D:\\product_price.xlsx";
        File file = new File(filePath);
        EasyExcel.read(file, ProductPriceExcel.class, productPriceExcelReadListener)
                .sheet().doRead();
    }



    @GetMapping("copyPrice")
    public void copyPrice() {
        productPriceService.copy();
    }

    @GetMapping("copyPriceV1")
    public void copyPriceV1() {
        productPriceService.copyV1();
    }



    @GetMapping("/countByProductIdAndTransportType")
    @ApiOperation("获得空运产品的路线价格数量")
    @ApiImplicitParam(name = "productId", value = "产品id", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Long> countByProductIdAndTransportType(@RequestParam("productId") Long productId) {
        Long count = productPriceService.countByProductIdAndTransportType(productId, CollectionUtil.newArrayList("3"));
        return success(count);
    }

    @GetMapping("/updateOrderException")
    public void updateOrderException(@RequestParam("priceId") Long priceId) {
        productPriceService.handleOrderException(priceId);
    }

    /**
     * 从商品列表中某一商品进入复制价格
     * @param dto
     *
     */
    @ApiOperation("从商品列表中某一商品进入复制价格")
    @PostMapping("/copyPriceByProductIds")
    public  void copyPriceByProductIds(@RequestBody CopyProductPriceDto dto ){
        productPriceService.copyPriceByProductIds(dto);
    }

}
