package cn.iocoder.yudao.module.product.controller.admin.product;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.module.product.vo.product.ProductTypeNumVO;
import cn.iocoder.yudao.module.product.convert.product.ProductTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductTypeDO;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.service.product.ProductTypeService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.product.vo.productType.*;

@Api(tags = "管理后台 - 商品类型")
@RestController
@RequestMapping("/ecw/product-type")
@Validated
public class ProductTypeController {

    @Resource
    private ProductTypeService productTypeService;
    @Resource
    private ProductService productService;

    @PostMapping("/create")
    @ApiOperation("创建商品类型")
    @PreAuthorize("@ss.hasPermission('ecw:product-type:create')")
    public CommonResult<Long> createProductType(@Valid @RequestBody ProductTypeCreateReqVO createReqVO) {
        return success(productTypeService.createProductType(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新商品类型")
    @PreAuthorize("@ss.hasPermission('ecw:product-type:update')")
    public CommonResult<Boolean> updateProductType(@Valid @RequestBody ProductTypeUpdateReqVO updateReqVO) {
        productTypeService.updateProductType(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除商品类型")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:product-type:delete')")
    public CommonResult<Boolean> deleteProductType(@RequestParam("id") Long id) {
        productTypeService.deleteProductType(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得产品类型")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:product-type:query')")
    public CommonResult<ProductTypeRespVO> getProductType(@RequestParam("id") Long id) {
        ProductTypeDO productType = productTypeService.getProductType(id);
        return success(ProductTypeConvert.INSTANCE.convert(productType));
    }

    @GetMapping("/list")
    @ApiOperation("获得商品类型列表")
//    @PreAuthorize("@ss.hasPermission('ecw:product-type:query')")
    public CommonResult<List<ProductTypeRespVO>> getProductTypeList(ProductTypeExportReqVO req) {
        List<ProductTypeDO> list = productTypeService.getProductTypeList(req);
        return success(ProductTypeConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得商品类型分页")
//    @PreAuthorize("@ss.hasPermission('ecw:product-type:query')")
    public CommonResult<PageResult<ProductTypeRespVO>> getProductTypePage(@Valid ProductTypePageReqVO pageVO) {
        PageResult<ProductTypeDO> pageResult = productTypeService.getProductTypePage(pageVO);
        //统计产品数量
        PageResult<ProductTypeRespVO> respPageResult = ProductTypeConvert.INSTANCE.convertPage(pageResult);
        List<ProductTypeRespVO> typeList = respPageResult.getList();
        List<Long> typeIdList = typeList.stream()
                .map(ProductTypeRespVO::getId)
                .collect(Collectors.toList());
        if(CollectionUtil.isNotEmpty(typeIdList)) {
            //统计产品数量
            List<ProductTypeNumVO> typeNumList = productService.getProductTypeNumList(typeIdList);
            Map<Long, Long> typeNumMap = typeNumList.stream()
                    .collect(Collectors.toMap(ProductTypeNumVO::getTypeId, ProductTypeNumVO::getCount));
            typeList.forEach(t -> {
                t.setProductNum(typeNumMap.get(t.getId()));
            });
        }
        return success(respPageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出商品类型 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:product-type:export')")
    @OperateLog(type = EXPORT)
    public void exportProductTypeExcel(@Valid ProductTypeExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<ProductTypeDO> list = productTypeService.getProductTypeList(exportReqVO);
        // 导出 Excel
        List<ProductTypeExcelVO> datas = ProductTypeConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "商品类型.xls", "数据", ProductTypeExcelVO.class, datas);
    }

}
