package cn.iocoder.yudao.module.product.controller.admin.productbrandempower;

import cn.iocoder.yudao.framework.common.util.spring.enums.CommonStatusEnum;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.product.vo.productbrandempower.*;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;

import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.product.dal.dataobject.productbrandempower.ProductBrandEmpowerDO;
import cn.iocoder.yudao.module.product.convert.productbrandempower.ProductBrandEmpowerConvert;
import cn.iocoder.yudao.module.product.service.productbrandempower.ProductBrandEmpowerService;

@Validated
@RestController
@Api(tags = "管理后台 - 商品品牌客户授权")
@RequestMapping("/ecw/product-brand-empower")
public class ProductBrandEmpowerController {

    @Resource
    private ProductBrandEmpowerService productBrandEmpowerService;

    @PostMapping("/create")
    @ApiOperation("创建品牌客户授权")
    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:create')")
    public CommonResult<Long> createProductBrandEmpower(@Valid @RequestBody ProductBrandEmpowerCreateReqVO createReqVO) {
        ProductBrandEmpowerBaseVO param = new ProductBrandEmpowerBaseVO();
        param.setCustomerId(createReqVO.getCustomerId());
        param.setProductBrandId(createReqVO.getProductBrandId());
        param.setStatus(CommonStatusEnum.ENABLE.getStatus());
        List list = productBrandEmpowerService.getProductBrandEmpowerList(param);
        if (list != null && list.size() > 0) {
            return error(ErrorCodeConstants.PRODUCT_BRAND_EMPOWER_EXISTS);
        }
        return success(productBrandEmpowerService.createProductBrandEmpower(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新品牌客户授权")
    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:update')")
    public CommonResult<Boolean> updateProductBrandEmpower(@Valid @RequestBody ProductBrandEmpowerUpdateReqVO updateReqVO) {
        productBrandEmpowerService.updateProductBrandEmpower(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除品牌客户授权")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:delete')")
    public CommonResult<Boolean> deleteProductBrandEmpower(@RequestParam("id") Long id) {
        productBrandEmpowerService.deleteProductBrandEmpower(id);
        return success(true);
    }

//    @GetMapping("/get")
//    @ApiOperation("获得品牌客户授权")
//    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:query')")
//    public CommonResult<ProductBrandEmpowerRespVO> getProductBrandEmpower(@RequestParam("id") Long id) {
//        ProductBrandEmpowerDO productBrandEmpower = productBrandEmpowerService.getProductBrandEmpower(id);
//        return success(ProductBrandEmpowerConvert.INSTANCE.convert(productBrandEmpower));
//    }




//    @GetMapping("/list")
//    @ApiOperation("获得品牌客户授权列表")
//    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:query')")
//    public CommonResult<List<ProductBrandEmpowerRespVO>> getProductBrandEmpowerList(@RequestParam("ids") Collection<Long> ids) {
//        List<ProductBrandEmpowerDO> list = productBrandEmpowerService.getProductBrandEmpowerList(ids);
//        return success(ProductBrandEmpowerConvert.INSTANCE.convertList(list));
//    }

    @GetMapping("/page/auth")
    @ApiOperation("获得品牌已授权客户分页")
    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:query')")
    public CommonResult<PageResult<ProductBrandEmpowerRespVO>> getProductBrandEmpowerAuthPage(@Valid ProductBrandEmpowerPageReqVO pageVO) {
        PageResult<ProductBrandEmpowerRespVO> pageResult = productBrandEmpowerService.getProductBrandEmpowerCustomerPage(pageVO);
        return success(pageResult);
    }

    @GetMapping("/page/unauth")
    @ApiOperation("获得品牌未授权客户分页")
    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:query')")
    public CommonResult<PageResult<ProductBrandEmpowerRespVO>> getProductBrandEmpowerUnauthPage(@Valid ProductBrandEmpowerPageReqVO pageVO) {
        PageResult<ProductBrandEmpowerRespVO> pageResult = productBrandEmpowerService.getProductBrandEmpowerUnAuthCustomerPage(pageVO);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出品牌客户授权 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:product-brand-empower:export')")
    @OperateLog(type = EXPORT)
    public void exportProductBrandEmpowerExcel(@Valid ProductBrandEmpowerExportReqVO exportReqVO,
                                               HttpServletResponse response) throws IOException {
        List<ProductBrandEmpowerDO> list = productBrandEmpowerService.getProductBrandEmpowerList(exportReqVO);
        // 导出 Excel
        List<ProductBrandEmpowerExcelVO> datas = ProductBrandEmpowerConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "商品品牌授权.xls", "数据", ProductBrandEmpowerExcelVO.class, datas);
    }
}
