package cn.iocoder.yudao.module.product.controller.app.coupon;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.product.dto.CouponUserAvailableDto;
import cn.iocoder.yudao.module.product.param.AvailableCouponParam;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import cn.iocoder.yudao.module.product.vo.coupon.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "app-web-优惠券信息")
@RestController
@RequestMapping("/product/coupon")
@Validated
public class AppCouponController {

    @Autowired
    private CouponService couponService;
    @Autowired
    private CustomerContactsService customerContactsService;


    @GetMapping("/get")
    @ApiOperation("获得优惠券信息")
    @ApiImplicitParam(name = "couponId", value = "优惠劵ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<CouponRespVO> getCoupon(@RequestParam("couponId") Long couponId) {
        CouponRespVO couponRespVO = couponService.getInfo(couponId);
        return success(couponRespVO);
    }


    @PostMapping("/customer/available/couponList")
    @ApiOperation("获得用户可用优惠信息")
    @PreAuthenticated
    @Deprecated
    public CommonResult<CouponUserAvailableDto> getCouponUserAvailableDto(@RequestBody @Valid AvailableCouponParam availableCouponParam) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        availableCouponParam.setUserId(userId);
        CustomerContactsDO contactsDO = customerContactsService.getOne(new LambdaQueryWrapper<CustomerContactsDO>()
                .eq(CustomerContactsDO::getDeleted, 0)
                .eq(CustomerContactsDO::getUserid, userId)
                .last("limit 1"));
        if (Objects.nonNull(contactsDO)) {
            availableCouponParam.setConsignorCustomerId(contactsDO.getCustomerId());
        }
        CouponUserAvailableDto dto = couponService.getCouponUserAvailableDto(availableCouponParam.getOrderType(),
                availableCouponParam.getCustomsType(), availableCouponParam.getIsCargoControl() ? 1 : 0, userId,
                availableCouponParam.getConsignorCustomerId(), null,
                availableCouponParam.getProdConditionParamList(), availableCouponParam.getTransportId(), availableCouponParam.getLineId(),
                availableCouponParam.getChannelId());
        return success(dto);
    }


}
