package cn.iocoder.yudao.module.product.controller.app.coupon;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.product.convert.coupon.CouponUseRecordConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponUseRecordDO;
import cn.iocoder.yudao.module.product.service.coupon.CouponUseRecordService;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordExcelVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordExportReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordPageReqVO;
import cn.iocoder.yudao.module.product.vo.coupon.record.CouponUseRecordRespVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Api(tags = "app-web-优惠券使用记录")
@RestController
@RequestMapping("/product/coupon-use-record")
@Validated
public class AppCouponUseRecordController {

    @Resource
    private CouponUseRecordService couponUseRecordService;


    @GetMapping("/info")
    @ApiOperation("获得优惠券使用记录详情")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<CouponUseRecordRespVO> getCouponUseRecord(@RequestParam("id") Long id) {
        CouponUseRecordDO couponUseRecord = couponUseRecordService.info(id);
        return success(CouponUseRecordConvert.INSTANCE.convert(couponUseRecord));
    }


    @GetMapping("/page")
    @ApiOperation("获得优惠券使用记录分页")
    @PreAuthenticated
    public CommonResult<PageResult<CouponUseRecordRespVO>> getCouponUseRecordPage(PageVO page, CouponUseRecordPageReqVO reqVO) {
        reqVO.setUserId(SecurityFrameworkUtils.getLoginUserId());
        PageResult<CouponUseRecordRespVO> pageResult = couponUseRecordService.couponUseRecordPage(page, reqVO);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出优惠券使用记录 Excel")
    @PreAuthenticated
    @OperateLog(type = EXPORT)
    public void exportCouponUseRecordExcel(@Valid CouponUseRecordExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<CouponUseRecordDO> list = couponUseRecordService.getCouponUseRecordList(exportReqVO);
        // 导出 Excel
        List<CouponUseRecordExcelVO> datas = CouponUseRecordConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "优惠券使用记录.xls", "数据", CouponUseRecordExcelVO.class, datas);
    }

}
