package cn.iocoder.yudao.module.product.controller.app.prod;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.product.component.ProdCostCalculation;
import cn.iocoder.yudao.module.product.dto.CalculationCostResultDto;
import cn.iocoder.yudao.module.product.dto.ProdFeeListDto;
import cn.iocoder.yudao.module.product.param.ConditionParam;
import cn.iocoder.yudao.module.product.param.ProdConditionParam;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.ORDER_ITEM_PROD_NOT_EXISTS;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.PROD_ID_NOT_NULL;

@Api(tags = "app - web - 商品线路价格费用计算")
@RestController
@RequestMapping("/product/line/price")
@Validated
public class AppProdLinePriceController {

    @Autowired
    private ProdCostCalculation prodCostCalculation;
    @Autowired
    private CouponService couponService;
    @Autowired
    private CustomerContactsService customerContactsService;


    /**
     * 计算单种运输方式的商品费用
     *
     * @param param 条件计费参数
     * @return 商品列表计费后信息
     */
    @PostMapping("/calculation")
    @ApiOperation("计算单种运输方式的商品费用(单个商品也做数组传参)")
    public CommonResult<CalculationCostResultDto> calculationProdCost(
            @Valid @RequestBody ConditionParam param) {
        if (CollectionUtil.isEmpty(param.getProdConditionParamList())){
            return error(ORDER_ITEM_PROD_NOT_EXISTS);
        }
        // TODO 前端实时计价时需要将订单的固定价格参数进行格式化，转为未定价，否则不会去查询当前实时价格
        List<ProdConditionParam> prodConditionParams = prodCostCalculation.obtainProdLineOnePrice(param.getProdConditionParamList(), "",
                Objects.nonNull(param.getCustomsType()) ? param.getCustomsType() : 1,
                null, param.getTransportId(), param.getLineId(), param.getChannelId(), false);
        CalculationCostResultDto dto =
                prodCostCalculation.calculationProdCost(prodConditionParams, "", param.getOrderType(),
                Objects.nonNull(param.getCustomsType()) ? param.getCustomsType() : 1, Objects.nonNull(param.getIsCargoControl()) && param.getIsCargoControl() ? 1 : 0,
                null, null, param.getConsignorCustomerId(), param.getConsigneeCustomerId(), param.getConsignorCustomerContactsId(),
                param.getConsigneeCustomerContactsId(), param.getTransportId(), param.getLineId(), param.getChannelId());
        return success(dto);
    }


    @PostMapping("/calculation/prod-fee-List")
    @ApiOperation("查询商品的价格列表(商品根据始发地和目的地获取所有线路得价格列表)")
    @ApiImplicitParam(name = "prodConditionParam", value = "商品信息", required = true, dataTypeClass = ProdConditionParam.class)
    public CommonResult<List<ProdFeeListDto>> calculationProdFeeList(@Valid @RequestBody ProdConditionParam prodConditionParam) throws IOException {
        if (null == prodConditionParam.getProdId()) {
            throw exception(PROD_ID_NOT_NULL);
        }
        List<ProdFeeListDto> prodFeeListVOList = prodCostCalculation.calculationProdFeeList(prodConditionParam.getDepartureId(), prodConditionParam.getObjectiveId(), prodConditionParam);
        return success(prodFeeListVOList);
    }

}
