package cn.iocoder.yudao.module.product.controller.app.product;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.product.vo.productAttr.*;
import cn.iocoder.yudao.module.product.convert.product.ProductAttrConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


@Api(tags = "app-web - 商品属性")
@RestController
@RequestMapping("/ecw/product-attr")
@Validated
public class AppProductAttrController {

    @Resource
    private ProductAttrService productAttrService;


    @GetMapping("/get")
    @ApiOperation("获得商品属性")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<ProductAttrRespVO> getProductAttr(@RequestParam("id") Long id) {
        ProductAttrDO productAttr = productAttrService.getProductAttr(id);
        return success(ProductAttrConvert.INSTANCE.convert(productAttr));
    }

    @GetMapping("/list")
    @ApiOperation("获得商品属性列表")
    @PreAuthenticated
    public CommonResult<List<ProductAttrRespVO>> getProductAttrList() {
        List<ProductAttrDO> list = productAttrService.getProductAttrList();
        return success(ProductAttrConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得商品属性分页")
    @PreAuthenticated
    public CommonResult<PageResult<ProductAttrRespVO>> getProductAttrPage(@Valid ProductAttrPageReqVO pageVO) {
        PageResult<ProductAttrDO> pageResult = productAttrService.getProductAttrPage(pageVO);
        return success(ProductAttrConvert.INSTANCE.convertPage(pageResult));
    }


}
