package cn.iocoder.yudao.module.product.controller.app.product;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.product.vo.productPrice.*;
import cn.iocoder.yudao.module.product.convert.product.ProductPriceConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceDO;
import cn.iocoder.yudao.module.product.service.product.ProductPriceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


@Api(tags = "app-web - 路线产品")
@RestController
@RequestMapping("/ecw/product-price")
@Validated
public class AppProductPriceController {

    @Resource
    private ProductPriceService productPriceService;


    @GetMapping("/get")
    @ApiOperation("获得路线产品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<ProductPriceRespVO> getProductPrice(@RequestParam("id") Long id) {
        ProductPriceRespVO productPriceRespVO = productPriceService.getProductPriceDetail(id);
        return success(productPriceRespVO);
    }

    @GetMapping("/list")
    @ApiOperation("获得路线产品列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthenticated
    public CommonResult<List<ProductPriceRespVO>> getProductPriceList(@RequestParam("ids") Collection<Long> ids) {
        List<ProductPriceDO> doList = productPriceService.getProductPriceList(ids);
        List<ProductPriceRespVO> list = ProductPriceConvert.INSTANCE.convertList(doList);
        productPriceService.matchLineAndProduct(list);
        return success(list);
    }

    @GetMapping("/page")
    @ApiOperation("获得路线产品分页")
    @PreAuthenticated
    public CommonResult<PageResult<ProductPriceRespVO>> getProductPricePage(@Valid ProductPricePageReqVO pageVO) {
        PageResult<ProductPriceDO> pageResult = productPriceService.getProductPricePage(pageVO);
        PageResult<ProductPriceRespVO> respPageResult = ProductPriceConvert.INSTANCE.convertPage(pageResult);
        productPriceService.matchLineAndProduct(respPageResult.getList());
        return success(respPageResult);
    }


}
