package cn.iocoder.yudao.module.product.controller.app.product;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.product.vo.product.ProductTypeNumVO;
import cn.iocoder.yudao.module.product.vo.productType.*;
import cn.iocoder.yudao.module.product.convert.product.ProductTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductTypeDO;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.service.product.ProductTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "app-web - 商品类型")
@RestController
@RequestMapping("/ecw/product-type")
@Validated
public class AppProductTypeController {

    @Resource
    private ProductTypeService productTypeService;
    @Resource
    private ProductService productService;


    @GetMapping("/get")
    @ApiOperation("获得产品类型")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<ProductTypeRespVO> getProductType(@RequestParam("id") Long id) {
        ProductTypeDO productType = productTypeService.getProductType(id);
        return success(ProductTypeConvert.INSTANCE.convert(productType));
    }

    @GetMapping("/list")
    @ApiOperation("获得商品类型列表")
    @PreAuthenticated
    public CommonResult<List<ProductTypeRespVO>> getProductTypeList(ProductTypeExportReqVO req) {
        List<ProductTypeDO> list = productTypeService.getProductTypeList(req);
        return success(ProductTypeConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得商品类型分页")
    @PreAuthenticated
    public CommonResult<PageResult<ProductTypeRespVO>> getProductTypePage(@Valid ProductTypePageReqVO pageVO) {
        PageResult<ProductTypeDO> pageResult = productTypeService.getProductTypePage(pageVO);
        //统计产品数量
        PageResult<ProductTypeRespVO> respPageResult = ProductTypeConvert.INSTANCE.convertPage(pageResult);
        List<ProductTypeRespVO> typeList = respPageResult.getList();
        List<Long> typeIdList = typeList.stream()
                .map(ProductTypeRespVO::getId)
                .collect(Collectors.toList());
        if(CollectionUtil.isNotEmpty(typeIdList)) {
            //统计产品数量
            List<ProductTypeNumVO> typeNumList = productService.getProductTypeNumList(typeIdList);
            Map<Long, Long> typeNumMap = typeNumList.stream()
                    .collect(Collectors.toMap(ProductTypeNumVO::getTypeId, ProductTypeNumVO::getCount));
            typeList.forEach(t -> {
                t.setProductNum(typeNumMap.get(t.getId()));
            });
        }
        return success(respPageResult);
    }

}
